@@ -27,8 +27,6 @@
  */
 
 require_once DOL_DOCUMENT_ROOT.'/core/class/commonobject.class.php';
-require_once DOL_DOCUMENT_ROOT.'/adherents/class/cotisation.class.php';
-require_once DOL_DOCUMENT_ROOT.'/core/lib/functions2.lib.php';
 require_once DOL_DOCUMENT_ROOT.'/core/lib/date.lib.php';
 
 
@@ -41,38 +39,33 @@ class Adherent extends CommonObject
     public $table_element='adherent';
     protected $ismultientitymanaged = 1;  // 0=No test on entity, 1=Test with field entity, 2=Test with link by societe
 
+    var $error;
+    var $errors;
+    var $mesgs;
+
     var $id;
     var $ref;
     var $civilite_id;
     var $firstname;
-    var $prenom;                // deprecated
     var $lastname;
-    var $nom;                   // deprecated
     var $login;
     var $pass;
     var $societe;
-    var $adresse;
+    var $company;
     var $address;
-    var $cp;
     var $zip;
-    var $ville;
     var $town;
 
     var $state_id;              // Id of department
     var $state_code;            // Code of department
     var $state;                 // Label of department
-    var $fk_departement;		// deprecated
-    var $departement_code;		// deprecated
-    var $departement;			// deprecated
 
     var $country_id;
     var $country_code;
     var $country;
-    var $pays_id;              // deprecated
-    var $pays_code;            // deprecated
-    var $pays;                 // deprecated
 
     var $email;
+    var $skype;
     var $phone;
     var $phone_perso;
     var $phone_mobile;
@@ -87,7 +80,7 @@ class Adherent extends CommonObject
     var $datem;
     var $datefin;
     var $datevalid;
-    var $naiss;
+    var $birth;
 
     var $typeid;			// Id type adherent
     var $type;				// Libelle type adherent
@@ -185,7 +178,7 @@ class Adherent extends CommonObject
 	{
 		global $conf,$langs;
 
-		$birthday = dol_print_date($this->naiss,'day');
+		$birthday = dol_print_date($this->birth,'day');
 
 		$msgishtml = 0;
 		if (dol_textishtml($text,1)) $msgishtml = 1;
@@ -224,16 +217,15 @@ class Adherent extends CommonObject
 				'%TOWN%'=>$msgishtml?dol_htmlentitiesbr($this->town):$this->town,
 				'%COUNTRY%'=>$msgishtml?dol_htmlentitiesbr($this->country):$this->country,
 				'%EMAIL%'=>$msgishtml?dol_htmlentitiesbr($this->email):$this->email,
-				'%NAISS%'=>$msgishtml?dol_htmlentitiesbr($birthday):$birthday,
+				'%BIRTH%'=>$msgishtml?dol_htmlentitiesbr($birthday):$birthday,
 				'%PHOTO%'=>$msgishtml?dol_htmlentitiesbr($this->photo):$this->photo,
 				'%LOGIN%'=>$msgishtml?dol_htmlentitiesbr($this->login):$this->login,
 				'%PASSWORD%'=>$msgishtml?dol_htmlentitiesbr($this->pass):$this->pass,
 				// For backward compatibility
 				'%INFOS%'=>$msgishtml?dol_htmlentitiesbr($infos):$infos,
+				'%SOCIETE%'=>$msgishtml?dol_htmlentitiesbr($this->societe):$this->societe,
 				'%PRENOM%'=>$msgishtml?dol_htmlentitiesbr($this->firstname):$this->firstname,
 				'%NOM%'=>$msgishtml?dol_htmlentitiesbr($this->lastname):$this->lastname,
-				'%SOCIETE%'=>$msgishtml?dol_htmlentitiesbr($this->societe):$this->societe,
-				'%ADRESSE%'=>$msgishtml?dol_htmlentitiesbr($this->address):$this->address,
 				'%CP%'=>$msgishtml?dol_htmlentitiesbr($this->zip):$this->zip,
 				'%VILLE%'=>$msgishtml?dol_htmlentitiesbr($this->town):$this->town,
 				'%PAYS%'=>$msgishtml?dol_htmlentitiesbr($this->country):$this->country,
@@ -275,6 +267,9 @@ class Adherent extends CommonObject
 
         $now=dol_now();
 
+        // Clean parameters
+        $this->import_key = trim($this->import_key);
+
         // Check parameters
         if (! empty($conf->global->ADHERENT_MAIL_REQUIRED) && ! isValidEMail($this->email))
         {
@@ -296,7 +291,7 @@ class Adherent extends CommonObject
 
         // Insert member
         $sql = "INSERT INTO ".MAIN_DB_PREFIX."adherent";
-        $sql.= " (datec,login,fk_user_author,fk_user_mod,fk_user_valid,morphy,fk_adherent_type,entity)";
+        $sql.= " (datec,login,fk_user_author,fk_user_mod,fk_user_valid,morphy,fk_adherent_type,entity,import_key)";
         $sql.= " VALUES (";
         $sql.= " '".$this->db->idate($this->datec)."'";
         $sql.= ", ".($this->login?"'".$this->db->escape($this->login)."'":"null");
@@ -304,6 +299,7 @@ class Adherent extends CommonObject
         $sql.= ", null, null, '".$this->morphy."'";
         $sql.= ", '".$this->typeid."'";
         $sql.= ", ".$conf->entity;
+        $sql.= ", ".(! empty($this->import_key) ? "'".$this->import_key."'":"null");
         $sql.= ")";
 
         dol_syslog(get_class($this)."::create sql=".$sql);
@@ -394,21 +390,21 @@ class Adherent extends CommonObject
      */
     function update($user,$notrigger=0,$nosyncuser=0,$nosyncuserpass=0,$nosyncthirdparty=0,$action='update')
     {
-        global $conf, $langs;
+        global $conf, $langs, $hookmanager;
 
         $nbrowsaffected=0;
         $error=0;
 
-        dol_syslog(get_class($this)."::update notrigger=".$notrigger.", nosyncuser=".$nosyncuser.", nosyncuserpass=".$nosyncuserpass.", email=".$this->email);
+        dol_syslog(get_class($this)."::update notrigger=".$notrigger.", nosyncuser=".$nosyncuser.", nosyncuserpass=".$nosyncuserpass." nosyncthirdparty=".$nosyncthirdparty.", email=".$this->email);
 
         // Clean parameters
-		$this->lastname=trim($this->lastname)?trim($this->lastname):trim($this->nom);
-		$this->firstname=trim($this->firstname)?trim($this->firstname):trim($this->prenom);
-		$this->address=($this->address?$this->address:$this->adresse);
-		$this->zip=($this->zip?$this->zip:$this->cp);
-		$this->town=($this->town?$this->town:$this->ville);
-		$this->country_id=($this->country_id > 0?$this->country_id:$this->fk_pays);
-		$this->state_id=($this->state_id > 0?$this->state_id:$this->fk_departement);
+		$this->lastname=trim($this->lastname)?trim($this->lastname):trim($this->lastname);
+		$this->firstname=trim($this->firstname)?trim($this->firstname):trim($this->firstname);
+		$this->address=($this->address?$this->address:$this->address);
+		$this->zip=($this->zip?$this->zip:$this->zip);
+		$this->town=($this->town?$this->town:$this->town);
+		$this->country_id=($this->country_id > 0?$this->country_id:$this->country_id);
+		$this->state_id=($this->state_id > 0?$this->state_id:$this->state_id);
 		if (! empty($conf->global->MAIN_FIRST_TO_UPPER)) $this->lastname=ucwords(trim($this->lastname));
         if (! empty($conf->global->MAIN_FIRST_TO_UPPER)) $this->firstname=ucwords(trim($this->firstname));
 
@@ -424,17 +420,18 @@ class Adherent extends CommonObject
 
         $sql = "UPDATE ".MAIN_DB_PREFIX."adherent SET";
         $sql.= " civilite = ".(!is_null($this->civilite_id)?"'".$this->civilite_id."'":"null");
-        $sql.= ", prenom = ".($this->firstname?"'".$this->db->escape($this->firstname)."'":"null");
-        $sql.= ", nom="     .($this->lastname?"'".$this->db->escape($this->lastname)."'":"null");
+        $sql.= ", firstname = ".($this->firstname?"'".$this->db->escape($this->firstname)."'":"null");
+        $sql.= ", lastname=" .($this->lastname?"'".$this->db->escape($this->lastname)."'":"null");
         $sql.= ", login="   .($this->login?"'".$this->db->escape($this->login)."'":"null");
         $sql.= ", societe=" .($this->societe?"'".$this->db->escape($this->societe)."'":"null");
         $sql.= ", fk_soc="  .($this->fk_soc > 0?"'".$this->fk_soc."'":"null");
-        $sql.= ", adresse=" .($this->address?"'".$this->db->escape($this->address)."'":"null");
-        $sql.= ", cp="      .($this->zip?"'".$this->db->escape($this->zip)."'":"null");
-        $sql.= ", ville="   .($this->town?"'".$this->db->escape($this->town)."'":"null");
-        $sql.= ", pays="          .($this->country_id>0?"'".$this->country_id."'":"null");
-        $sql.= ", fk_departement=".($this->state_id>0?"'".$this->state_id."'":"null");
+        $sql.= ", address=" .($this->address?"'".$this->db->escape($this->address)."'":"null");
+        $sql.= ", zip="      .($this->zip?"'".$this->db->escape($this->zip)."'":"null");
+        $sql.= ", town="   .($this->town?"'".$this->db->escape($this->town)."'":"null");
+        $sql.= ", country=".($this->country_id>0?"'".$this->country_id."'":"null");
+        $sql.= ", state_id=".($this->state_id>0?"'".$this->state_id."'":"null");
         $sql.= ", email='".$this->email."'";
+        $sql.= ", skype='".$this->skype."'";
         $sql.= ", phone="   .($this->phone?"'".$this->db->escape($this->phone)."'":"null");
         $sql.= ", phone_perso="  .($this->phone_perso?"'".$this->db->escape($this->phone_perso)."'":"null");
         $sql.= ", phone_mobile=" .($this->phone_mobile?"'".$this->db->escape($this->phone_mobile)."'":"null");
@@ -444,7 +441,7 @@ class Adherent extends CommonObject
         $sql.= ", statut="  .$this->statut;
         $sql.= ", fk_adherent_type=".$this->typeid;
         $sql.= ", morphy='".$this->morphy."'";
-        $sql.= ", naiss="   .($this->naiss?"'".$this->db->idate($this->naiss)."'":"null");
+        $sql.= ", birth="   .($this->birth?"'".$this->db->idate($this->birth)."'":"null");
         if ($this->datefin)   $sql.= ", datefin='".$this->db->idate($this->datefin)."'";		// Ne doit etre modifie que par effacement cotisation
         if ($this->datevalid) $sql.= ", datevalid='".$this->db->idate($this->datevalid)."'";	// Ne doit etre modifie que par validation adherent
         $sql.= ", fk_user_mod=".($user->id>0?$user->id:'null');	// Can be null because member can be create by a guest
@@ -462,8 +459,6 @@ class Adherent extends CommonObject
 		    $nbrowsaffected+=$this->db->affected_rows($resql);
 
             // Actions on extra fields (by external module)
-            include_once DOL_DOCUMENT_ROOT.'/core/class/hookmanager.class.php';
-            $hookmanager=new HookManager($this->db);
             $hookmanager->initHooks(array('memberdao'));
             $parameters=array('id'=>$this->id);
             $action='';
@@ -528,13 +523,12 @@ class Adherent extends CommonObject
                         $luser->civilite_id=$this->civilite_id;
                         $luser->firstname=$this->firstname;
                         $luser->lastname=$this->lastname;
-                        $luser->prenom=$this->firstname;    // deprecated
-                        $luser->nom=$this->lastname;        // deprecated
                         $luser->login=$this->user_login;
                         $luser->pass=$this->pass;
                         $luser->societe_id=$this->societe;
 
                         $luser->email=$this->email;
+                        $luser->skype=$this->skype;
                         $luser->office_phone=$this->phone;
                         $luser->user_mobile=$this->phone_mobile;
 
@@ -573,10 +567,11 @@ class Adherent extends CommonObject
                         $lthirdparty->zip=$this->zip;
                         $lthirdparty->town=$this->town;
                         $lthirdparty->email=$this->email;
-                        $lthirdparty->tel=$this->phone;
+                        $lthirdparty->skype=$this->skype;
+                        $lthirdparty->phone=$this->phone;
                         $lthirdparty->state_id=$this->state_id;
                         $lthirdparty->country_id=$this->country_id;
-                        $lthirdparty->pays_id=$this->country_id;
+                        $lthirdparty->country_id=$this->country_id;
                         //$lthirdparty->phone_mobile=$this->phone_mobile;
 
                         $result=$lthirdparty->update($this->fk_soc,$user,0,1,1,'update');	// Use sync to 0 to avoid cyclic updates
@@ -701,6 +696,9 @@ class Adherent extends CommonObject
 		$error=0;
 		$errorflag=0;
 
+		// Check parameters
+		if (empty($rowid)) $rowid=$this->id;
+
         $this->db->begin();
 
         // Remove category
@@ -948,7 +946,7 @@ class Adherent extends CommonObject
 
         $this->db->begin();
 
-        // Update link to third party
+        // Remove link to third party onto any other members
         if ($thirdpartyid > 0)
         {
             $sql = "UPDATE ".MAIN_DB_PREFIX."adherent SET fk_soc = null";
@@ -958,7 +956,7 @@ class Adherent extends CommonObject
             $resql = $this->db->query($sql);
         }
 
-        // Update link to third party
+        // Add link to third party for current member
         $sql = "UPDATE ".MAIN_DB_PREFIX."adherent SET fk_soc = ".($thirdpartyid>0 ? $thirdpartyid : 'null');
         $sql.= " WHERE rowid = ".$this->id;
 
@@ -1008,6 +1006,36 @@ class Adherent extends CommonObject
         }
     }
 
+    /**
+     *	Method to load member from its name
+     *
+     *	@param	string	$firstname	Firstname
+     *	@param	string	$lastname	Lastname
+     *	@return	void
+     */
+    function fetch_name($firstname,$lastname)
+    {
+    	global $conf;
+
+    	$sql = "SELECT rowid FROM ".MAIN_DB_PREFIX."adherent";
+    	$sql.= " WHERE firstname='".$this->db->escape($firstname)."'";
+    	$sql.= " AND lastname='".$this->db->escape($lastname)."'";
+    	$sql.= " AND entity = ".$conf->entity;
+
+    	$resql=$this->db->query($sql);
+    	if ($resql)
+    	{
+    		if ($this->db->num_rows($resql))
+    		{
+    			$obj = $this->db->fetch_object($resql);
+    			$this->fetch($obj->rowid);
+    		}
+    	}
+    	else
+    	{
+    		dol_print_error($this->db);
+    	}
+    }
 
     /**
      *	Load member from database
@@ -1015,29 +1043,30 @@ class Adherent extends CommonObject
      *	@param	int		$rowid      Id of object to load
      * 	@param	string	$ref		To load member from its ref
      * 	@param	int		$fk_soc		To load member from its link to third party
+     * 	@param	int		$ref_ext	External reference
      *	@return int         		>0 if OK, 0 if not found, <0 if KO
      */
-    function fetch($rowid,$ref='',$fk_soc='')
+    function fetch($rowid,$ref='',$fk_soc='',$ref_ext='')
     {
         global $langs;
 
-        $sql = "SELECT d.rowid, d.civilite, d.prenom as firstname, d.nom as lastname, d.societe, d.fk_soc, d.statut, d.public, d.adresse as address, d.cp as zip, d.ville as town, d.note,";
-        $sql.= " d.email, d.phone, d.phone_perso, d.phone_mobile, d.login, d.pass,";
+        $sql = "SELECT d.rowid, d.ref_ext, d.civilite, d.firstname, d.lastname, d.societe as company, d.fk_soc, d.statut, d.public, d.address, d.zip, d.town, d.note,";
+        $sql.= " d.email, d.skype, d.phone, d.phone_perso, d.phone_mobile, d.login, d.pass,";
         $sql.= " d.photo, d.fk_adherent_type, d.morphy, d.entity,";
         $sql.= " d.datec as datec,";
         $sql.= " d.tms as datem,";
         $sql.= " d.datefin as datefin,";
-        $sql.= " d.naiss as datenaiss,";
+        $sql.= " d.birth as birthday,";
         $sql.= " d.datevalid as datev,";
-        $sql.= " d.pays,";
-        $sql.= " d.fk_departement,";
+        $sql.= " d.country,";
+        $sql.= " d.state_id,";
         $sql.= " p.rowid as country_id, p.code as country_code, p.libelle as country,";
         $sql.= " dep.nom as state, dep.code_departement as state_code,";
         $sql.= " t.libelle as type, t.cotisation as cotisation,";
         $sql.= " u.rowid as user_id, u.login as user_login";
         $sql.= " FROM ".MAIN_DB_PREFIX."adherent_type as t, ".MAIN_DB_PREFIX."adherent as d";
-        $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_pays as p ON d.pays = p.rowid";
-        $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_departements as dep ON d.fk_departement = dep.rowid";
+        $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_pays as p ON d.country = p.rowid";
+        $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_departements as dep ON d.state_id = dep.rowid";
         $sql.= " LEFT JOIN ".MAIN_DB_PREFIX."user as u ON d.rowid = u.fk_member";
         $sql.= " WHERE d.fk_adherent_type = t.rowid";
         if ($rowid) $sql.= " AND d.rowid=".$rowid;
@@ -1046,6 +1075,10 @@ class Adherent extends CommonObject
         	if ($ref) $sql.= " AND d.rowid='".$ref."'";
         	elseif ($fk_soc) $sql.= " AND d.fk_soc='".$fk_soc."'";
         }
+        elseif ($ref_ext)
+        {
+        	$sql.= " AND d.ref_ext='".$this->db->escape($ref_ext)."'";
+        }
 
         dol_syslog(get_class($this)."::fetch sql=".$sql);
         $resql=$this->db->query($sql);
@@ -1058,28 +1091,22 @@ class Adherent extends CommonObject
                 $this->entity			= $obj->entity;
                 $this->ref				= $obj->rowid;
                 $this->id				= $obj->rowid;
+                $this->ref_ext			= $obj->ref_ext;
                 $this->civilite_id		= $obj->civilite;
-                $this->prenom			= $obj->firstname;   // deprecated
                 $this->firstname		= $obj->firstname;
-                $this->nom				= $obj->lastname;    // deprecated
                 $this->lastname			= $obj->lastname;
                 $this->login			= $obj->login;
                 $this->pass				= $obj->pass;
-                $this->societe			= $obj->societe;
+                $this->societe			= $obj->company;
+                $this->company			= $obj->company;
                 $this->fk_soc			= $obj->fk_soc;
-                $this->adresse			= $obj->address;	// deprecated
                 $this->address			= $obj->address;
-                $this->cp				= $obj->zip;		// deprecated
                 $this->zip				= $obj->zip;
-                $this->ville			= $obj->town;	    // deprecated
                 $this->town				= $obj->town;
 
-                $this->state_id			= $obj->fk_departement;
-                $this->state_code		= $obj->fk_departement?$obj->state_code:'';
-                $this->state			= $obj->fk_departement?$obj->state:'';
-                $this->fk_departement	= $obj->fk_departement;                        // deprecated
-                $this->departement_code	= $obj->fk_departement?$obj->state_code:'';    // deprecated
-                $this->departement		= $obj->fk_departement?$obj->state:'';         // deprecated
+                $this->state_id			= $obj->state_id;
+                $this->state_code		= $obj->state_id?$obj->state_code:'';
+                $this->state			= $obj->state_id?$obj->state:'';
 
                 $this->country_id		= $obj->country_id;
                 $this->country_code		= $obj->country_code;
@@ -1087,14 +1114,12 @@ class Adherent extends CommonObject
                 	$this->country = $langs->transnoentitiesnoconv("Country".$obj->country_code);
                 else
                 	$this->country=$obj->country;
-                $this->pays_id			= $obj->country_id;      // deprecated
-                $this->pays_code		= $obj->country_code;    // deprecated
-                $this->pays				= $this->country;        // deprecated
 
                 $this->phone			= $obj->phone;
                 $this->phone_perso		= $obj->phone_perso;
                 $this->phone_mobile		= $obj->phone_mobile;
                 $this->email			= $obj->email;
+                $this->skype			= $obj->skype;
 
                 $this->photo			= $obj->photo;
                 $this->statut			= $obj->statut;
@@ -1104,7 +1129,7 @@ class Adherent extends CommonObject
                 $this->datem			= $this->db->jdate($obj->datem);
                 $this->datefin			= $this->db->jdate($obj->datefin);
                 $this->datevalid		= $this->db->jdate($obj->datev);
-                $this->naiss			= $this->db->jdate($obj->datenaiss);
+                $this->birth			= $this->db->jdate($obj->birthday);
 
                 $this->note				= $obj->note;
                 $this->morphy			= $obj->morphy;
@@ -1116,6 +1141,13 @@ class Adherent extends CommonObject
                 $this->user_id			= $obj->user_id;
                 $this->user_login		= $obj->user_login;
 
+                // Retreive all extrafield for thirdparty
+                // fetch optionals attributes and labels
+                require_once DOL_DOCUMENT_ROOT.'/core/class/extrafields.class.php';
+                $extrafields=new ExtraFields($this->db);
+                $extralabels=$extrafields->fetch_name_optionals_label($this->table_element,true);
+                $this->fetch_optionals($this->id,$extralabels);
+
                 // Load other properties
                 $result=$this->fetch_subscriptions();
 
@@ -1148,6 +1180,8 @@ class Adherent extends CommonObject
     {
         global $langs;
 
+		require_once DOL_DOCUMENT_ROOT.'/adherents/class/cotisation.class.php';
+
         $sql = "SELECT c.rowid, c.fk_adherent, c.cotisation, c.note, c.fk_bank,";
         $sql.= " c.tms as datem,";
         $sql.= " c.datec as datec,";
@@ -1203,7 +1237,7 @@ class Adherent extends CommonObject
      *	Insert subscription into database and eventually add links to banks, mailman, etc...
      *
      *	@param	timestamp	$date        		Date d'effet de la cotisation
-     *	@param	amount		$montant     		Montant cotisation (accepte 0 pour les adherents non soumis e cotisation)
+     *	@param	amount		$montant     		Montant cotisation (accepte 0 pour les adherents non soumis a cotisation)
      *	@param	int			$accountid			Id compte bancaire
      *	@param	string		$operation			Type operation (si Id compte bancaire fourni)
      *	@param	string		$label				Label operation (si Id compte bancaire fourni)
@@ -1217,6 +1251,8 @@ class Adherent extends CommonObject
     {
         global $conf,$langs,$user;
 
+		require_once DOL_DOCUMENT_ROOT.'/adherents/class/cotisation.class.php';
+
 		$error=0;
 
         // Clean parameters
@@ -1308,7 +1344,7 @@ class Adherent extends CommonObject
 
         $sql = "UPDATE ".MAIN_DB_PREFIX."adherent SET";
         $sql.= " statut = 1";
-        $sql.= ", datevalid = ".$this->db->idate($now);
+        $sql.= ", datevalid = '".$this->db->idate($now)."'";
         $sql.= ", fk_user_valid=".$user->id;
         $sql.= " WHERE rowid = ".$this->id;
 
@@ -1388,17 +1424,16 @@ class Adherent extends CommonObject
 
 
     /**
-     *  Fonction qui ajoute l'adherent au abonnements automatiques mailing-list, spip, etc.
-     *  TODO Move this into member creation trigger (trigger of mailmanspip module)
+     *  Function to add member into external tools mailing-list, spip, etc.
      *
      *  @return		int		<0 if KO, >0 if OK
      */
     function add_to_abo()
     {
-        global $conf;
+        global $conf,$langs;
 
         include_once DOL_DOCUMENT_ROOT.'/mailmanspip/class/mailmanspip.class.php';
-        $mailmanspip=new MailmanSpip($db);
+        $mailmanspip=new MailmanSpip($this->db);
 
         $err=0;
 
@@ -1408,9 +1443,19 @@ class Adherent extends CommonObject
             $result=$mailmanspip->add_to_mailman($this);
             if ($result < 0)
             {
-            	$this->error=$mailmanspip->error;
+            	if (! empty($mailmanspip->error)) $this->errors[]=$mailmanspip->error;
                 $err+=1;
             }
+            foreach ($mailmanspip->mladded_ko as $tmplist => $tmpemail)
+            {
+            	$langs->load("errors");
+            	$this->errors[]=$langs->trans("ErrorFailedToAddToMailmanList",$tmpemail,$tmplist);
+            }
+            foreach ($mailmanspip->mladded_ok as $tmplist => $tmpemail)
+            {
+            	$langs->load("mailmanspip");
+            	$this->mesgs[]=$langs->trans("SuccessToAddToMailmanList",$tmpemail,$tmplist);
+            }
         }
 
         // spip
@@ -1419,34 +1464,32 @@ class Adherent extends CommonObject
             $result=$mailmanspip->add_to_spip($this);
             if ($result < 0)
             {
-            	$this->error=$mailmanspip->error;
+            	$this->errors[]=$mailmanspip->error;
             	$err+=1;
             }
         }
         if ($err)
         {
-            // error
             return -$err;
         }
         else
-        {
+       {
             return 1;
         }
     }
 
 
     /**
-     *  Fonction qui supprime l'adherent des abonnements automatiques mailing-list, spip, etc.
-     *  TODO Move this into member deletion trigger (trigger of mailmanspip module)
+     *  Function to delete a member from external tools like mailing-list, spip, etc.
      *
      *  @return     int     <0 if KO, >0 if OK
      */
     function del_to_abo()
     {
-        global $conf;
+        global $conf,$langs;
 
         include_once DOL_DOCUMENT_ROOT.'/mailmanspip/class/mailmanspip.class.php';
-        $mailmanspip=new MailmanSpip($db);
+        $mailmanspip=new MailmanSpip($this->db);
 
         $err=0;
 
@@ -1456,8 +1499,20 @@ class Adherent extends CommonObject
             $result=$mailmanspip->del_to_mailman($this);
             if ($result < 0)
             {
+                if (! empty($mailmanspip->error)) $this->errors[]=$mailmanspip->error;
                 $err+=1;
             }
+
+            foreach ($mailmanspip->mlremoved_ko as $tmplist => $tmpemail)
+            {
+            	$langs->load("errors");
+            	$this->errors[]=$langs->trans("ErrorFailedToRemoveToMailmanList",$tmpemail,$tmplist);
+            }
+            foreach ($mailmanspip->mlremoved_ok as $tmplist => $tmpemail)
+            {
+            	$langs->load("mailmanspip");
+            	$this->mesgs[]=$langs->trans("SuccessToRemoveToMailmanList",$tmpemail,$tmplist);
+            }
         }
 
         if ($conf->global->ADHERENT_USE_SPIP && ! empty($conf->mailmanspip->enabled))
@@ -1465,6 +1520,7 @@ class Adherent extends CommonObject
             $result=$mailmanspip->del_to_spip($this);
             if ($result < 0)
             {
+            	$this->errors[]=$mailmanspip->error;
                 $err+=1;
             }
         }
@@ -1481,19 +1537,18 @@ class Adherent extends CommonObject
 
 
     /**
-     *    Return label of a civility of a contact
+     *    Return civility label of a member
      *
-     *    @param	int		$nohtmlentities     0=Encode with htmlentities for HTML output, 1=No htmlentities for memory translation
-     *    @return   string              		Name translated of civility
+     *    @return   string              	Translated name of civility
      */
-    function getCivilityLabel($nohtmlentities=0)
+    function getCivilityLabel()
     {
-        global $langs;
-        $langs->load("dict");
+    	global $langs;
+    	$langs->load("dict");
 
-        $code=$this->civilite_id;
-        if ($nohtmlentities) return $langs->transnoentities("Civility".$code)!="Civility".$code ? $langs->transnoentities("Civility".$code) : $code;
-        else return $langs->trans("Civility".$code)!="Civility".$code ? $langs->trans("Civility".$code) : $code;
+    	$code=(! empty($this->civilite_id)?$this->civilite_id:(! empty($this->civility_id)?$this->civility_id:''));
+    	if (empty($code)) return '';
+    	return $langs->getLabelFromKey($this->db, "Civility".$code, "c_civilite", "code", "civilite", $code);
     }
 
     /**
@@ -1520,6 +1575,11 @@ class Adherent extends CommonObject
             $lien = '<a href="'.DOL_URL_ROOT.'/adherents/card_subscriptions.php?rowid='.$this->id.'">';
             $lienfin='</a>';
         }
+        if ($option == 'category')
+        {
+        	$lien = '<a href="'.DOL_URL_ROOT.'/categories/categorie.php?id='.$this->id.'&type=3">';
+        	$lienfin='</a>';
+        }
 
         $picto='user';
         $label=$langs->trans("ShowMember");
@@ -1530,40 +1590,6 @@ class Adherent extends CommonObject
         return $result;
     }
 
-
-    /**
-     * 	Return full address of member
-     *
-     * 	@param		int			$withcountry		1=Add country into address string
-     *  @param		string		$sep				Separator to use to build string
-     *	@return		string							Full address string
-     */
-    function getFullAddress($withcountry=0,$sep="\n")
-    {
-        $ret='';
-        if ($withcountry && $this->country_id && (empty($this->country_code) || empty($this->country)))
-        {
-            require_once DOL_DOCUMENT_ROOT .'/core/lib/company.lib.php';
-            $tmparray=getCountry($this->country_id,'all');
-            $this->country_code=$tmparray['code'];
-            $this->country     =$tmparray['label'];
-        }
-
-        if (in_array($this->country_code,array('US')))
-        {
-	        $ret.=($this->address?$this->address.$sep:'');
-	        $ret.=trim($this->zip.' '.$this->town);
-	        if ($withcountry) $ret.=($this->country?$sep.$this->country:'');
-        }
-        else
-        {
-	        $ret.=($this->address?$this->address.$sep:'');
-	        $ret.=trim($this->zip.' '.$this->town);
-	        if ($withcountry) $ret.=($this->country?$sep.$this->country:'');
-        }
-        return trim($ret);
-    }
-
     /**
      *  Retourne le libelle du statut d'un adherent (brouillon, valide, resilie)
      *
@@ -1648,11 +1674,11 @@ class Adherent extends CommonObject
             if ($statut == -1) return $langs->trans("MemberStatusDraft").' '.img_picto($langs->trans('MemberStatusDraft'),'statut0');
             if ($statut >= 1)
             {
-                if (! $date_end_subscription)            return $langs->trans("MemberStatusActive").' '.img_picto($langs->trans('MemberStatusActive'),'statut1');
-                elseif ($date_end_subscription < time()) return $langs->trans("MemberStatusActiveLate").' '.img_picto($langs->trans('MemberStatusActiveLate'),'statut3');
-                else                                     return $langs->trans("MemberStatusPaid").' '.img_picto($langs->trans('MemberStatusPaid'),'statut4');
+                if (! $date_end_subscription)            return '<span class="hideonsmartphone">'.$langs->trans("MemberStatusActive").' </span>'.img_picto($langs->trans('MemberStatusActive'),'statut1');
+                elseif ($date_end_subscription < time()) return '<span class="hideonsmartphone">'.$langs->trans("MemberStatusActiveLate").' </span>'.img_picto($langs->trans('MemberStatusActiveLate'),'statut3');
+                else                                     return '<span class="hideonsmartphone">'.$langs->trans("MemberStatusPaid").' </span>'.img_picto($langs->trans('MemberStatusPaid'),'statut4');
             }
-            if ($statut == 0)  return $langs->trans("MemberStatusResiliated").' '.img_picto($langs->trans('MemberStatusResiliated'),'statut5');
+            if ($statut == 0)  return '<span class="hideonsmartphone">'.$langs->trans("MemberStatusResiliated").' </span>'.img_picto($langs->trans('MemberStatusResiliated'),'statut5');
         }
     }
 
@@ -1680,6 +1706,7 @@ class Adherent extends CommonObject
             {
                 $this->nb["members"]=$obj->nb;
             }
+            $this->db->free($resql);
             return 1;
         }
         else
@@ -1760,11 +1787,12 @@ class Adherent extends CommonObject
         $this->country = 'France';
         $this->morphy = 1;
         $this->email = 'specimen@specimen.com';
+        $this->skype = 'tom.hanson';
         $this->phone        = '0999999999';
         $this->phone_perso  = '0999999998';
         $this->phone_mobile = '0999999997';
         $this->note='No comment';
-        $this->naiss=time();
+        $this->birth=time();
         $this->photo='';
         $this->public=1;
         $this->statut=0;
@@ -1821,22 +1849,23 @@ class Adherent extends CommonObject
 
         // Member
         if ($this->fullname && ! empty($conf->global->LDAP_MEMBER_FIELD_FULLNAME)) $info[$conf->global->LDAP_MEMBER_FIELD_FULLNAME] = $this->fullname;
-        if ($this->nom && ! empty($conf->global->LDAP_MEMBER_FIELD_NAME))         $info[$conf->global->LDAP_MEMBER_FIELD_NAME] = $this->nom;
-        if ($this->prenom && ! empty($conf->global->LDAP_MEMBER_FIELD_FIRSTNAME)) $info[$conf->global->LDAP_MEMBER_FIELD_FIRSTNAME] = $this->prenom;
+        if ($this->lastname && ! empty($conf->global->LDAP_MEMBER_FIELD_NAME))     $info[$conf->global->LDAP_MEMBER_FIELD_NAME] = $this->lastname;
+        if ($this->firstname && ! empty($conf->global->LDAP_MEMBER_FIELD_FIRSTNAME)) $info[$conf->global->LDAP_MEMBER_FIELD_FIRSTNAME] = $this->firstname;
         if ($this->login && ! empty($conf->global->LDAP_MEMBER_FIELD_LOGIN))      $info[$conf->global->LDAP_MEMBER_FIELD_LOGIN] = $this->login;
         if ($this->pass && ! empty($conf->global->LDAP_MEMBER_FIELD_PASSWORD))    $info[$conf->global->LDAP_MEMBER_FIELD_PASSWORD] = $this->pass;	// this->pass = mot de passe non crypte
         if ($this->poste && ! empty($conf->global->LDAP_MEMBER_FIELD_TITLE))      $info[$conf->global->LDAP_MEMBER_FIELD_TITLE] = $this->poste;
-        if ($this->adresse && ! empty($conf->global->LDAP_MEMBER_FIELD_ADDRESS))  $info[$conf->global->LDAP_MEMBER_FIELD_ADDRESS] = $this->adresse;
-        if ($this->cp && ! empty($conf->global->LDAP_MEMBER_FIELD_ZIP))           $info[$conf->global->LDAP_MEMBER_FIELD_ZIP] = $this->cp;
-        if ($this->ville && ! empty($conf->global->LDAP_MEMBER_FIELD_TOWN))       $info[$conf->global->LDAP_MEMBER_FIELD_TOWN] = $this->ville;
+        if ($this->address && ! empty($conf->global->LDAP_MEMBER_FIELD_ADDRESS))  $info[$conf->global->LDAP_MEMBER_FIELD_ADDRESS] = $this->address;
+        if ($this->zip && ! empty($conf->global->LDAP_MEMBER_FIELD_ZIP))           $info[$conf->global->LDAP_MEMBER_FIELD_ZIP] = $this->zip;
+        if ($this->town && ! empty($conf->global->LDAP_MEMBER_FIELD_TOWN))        $info[$conf->global->LDAP_MEMBER_FIELD_TOWN] = $this->town;
         if ($this->country_code && ! empty($conf->global->LDAP_MEMBER_FIELD_COUNTRY))     $info[$conf->global->LDAP_MEMBER_FIELD_COUNTRY] = $this->country_code;
         if ($this->email && ! empty($conf->global->LDAP_MEMBER_FIELD_MAIL))       $info[$conf->global->LDAP_MEMBER_FIELD_MAIL] = $this->email;
+        if ($this->skype && ! empty($conf->global->LDAP_MEMBER_FIELD_SKYPE))       $info[$conf->global->LDAP_MEMBER_FIELD_SKYPE] = $this->skype;
         if ($this->phone && ! empty($conf->global->LDAP_MEMBER_FIELD_PHONE))      $info[$conf->global->LDAP_MEMBER_FIELD_PHONE] = $this->phone;
         if ($this->phone_perso && ! empty($conf->global->LDAP_MEMBER_FIELD_PHONE_PERSO)) $info[$conf->global->LDAP_MEMBER_FIELD_PHONE_PERSO] = $this->phone_perso;
         if ($this->phone_mobile && ! empty($conf->global->LDAP_MEMBER_FIELD_MOBILE)) $info[$conf->global->LDAP_MEMBER_FIELD_MOBILE] = $this->phone_mobile;
         if ($this->fax && ! empty($conf->global->LDAP_MEMBER_FIELD_FAX))	      $info[$conf->global->LDAP_MEMBER_FIELD_FAX] = $this->fax;
         if ($this->note && ! empty($conf->global->LDAP_MEMBER_FIELD_DESCRIPTION)) $info[$conf->global->LDAP_MEMBER_FIELD_DESCRIPTION] = $this->note;
-        if ($this->naiss && ! empty($conf->global->LDAP_MEMBER_FIELD_BIRTHDATE))  $info[$conf->global->LDAP_MEMBER_FIELD_BIRTHDATE] = dol_print_date($this->naiss,'dayhourldap');
+        if ($this->birth && ! empty($conf->global->LDAP_MEMBER_FIELD_BIRTHDATE))  $info[$conf->global->LDAP_MEMBER_FIELD_BIRTHDATE] = dol_print_date($this->birth,'dayhourldap');
         if (isset($this->statut) && ! empty($conf->global->LDAP_FIELD_MEMBER_STATUS))  $info[$conf->global->LDAP_FIELD_MEMBER_STATUS] = $this->statut;
         if ($this->datefin && ! empty($conf->global->LDAP_FIELD_MEMBER_END_LASTSUBSCRIPTION))  $info[$conf->global->LDAP_FIELD_MEMBER_END_LASTSUBSCRIPTION] = dol_print_date($this->datefin,'dayhourldap');
 
