@@ -1,6 +1,6 @@
 <?php
 /* Copyright (C) 2001-2005 Rodolphe Quiedeville   	<rodolphe@quiedeville.org>
- * Copyright (C) 2004-2011 Laurent Destailleur   	<eldy@users.sourceforge.net>
+ * Copyright (C) 2004-2013 Laurent Destailleur   	<eldy@users.sourceforge.net>
  * Copyright (C) 2005      Marc Barilley / Ocebo  	<marc@ocebo.com>
  * Copyright (C) 2005-2012 Regis Houssin          	<regis.houssin@capnetworks.com>
  * Copyright (C) 2012	   Andreu Bisquerra Gaya  	<jove@bisquerra.com>
@@ -34,7 +34,9 @@ require_once DOL_DOCUMENT_ROOT.'/core/class/html.formfile.class.php';
 require_once DOL_DOCUMENT_ROOT.'/core/class/html.formother.class.php';
 require_once DOL_DOCUMENT_ROOT.'/core/lib/date.lib.php';
 require_once DOL_DOCUMENT_ROOT.'/core/lib/report.lib.php';
-if (! empty($conf->projet->enabled)) require_once DOL_DOCUMENT_ROOT.'/core/lib/project.lib.php';
+if (! empty($conf->projet->enabled)) {
+	require_once DOL_DOCUMENT_ROOT.'/core/class/html.formprojet.class.php';
+}
 
 $langs->load('orders');
 $langs->load('deliveries');
@@ -78,6 +80,12 @@ if ($action == 'create')
 	}
 }
 
+// Initialize technical object to manage hooks of thirdparties. Note that conf->hooks_modules contains array array
+include_once DOL_DOCUMENT_ROOT.'/core/class/hookmanager.class.php';
+$hookmanager=new HookManager($db);
+$hookmanager->initHooks(array('orderstoinvoice'));
+
+
 /*
  * Actions
  */
@@ -88,9 +96,8 @@ if (($action == 'create' || $action == 'add') && empty($mesgs))
 	require_once DOL_DOCUMENT_ROOT.'/core/class/discount.class.php';
 	require_once DOL_DOCUMENT_ROOT.'/core/lib/functions2.lib.php';
 	require_once DOL_DOCUMENT_ROOT.'/core/lib/invoice.lib.php';
-	if (! empty($conf->projet->enabled)) {
-		require_once DOL_DOCUMENT_ROOT.'/projet/class/project.class.php';
-	}
+	if (! empty($conf->projet->enabled)) require_once DOL_DOCUMENT_ROOT.'/projet/class/project.class.php';
+	
 	$langs->load('bills');
 	$langs->load('products');
 	$langs->load('main');
@@ -119,7 +126,7 @@ if (($action == 'create' || $action == 'add') && empty($mesgs))
 	$lineid			= GETPOST('lineid','int');
 	$userid			= GETPOST('userid','int');
 	$search_ref		= GETPOST('sf_ref')?GETPOST('sf_ref'):GETPOST('search_ref');
-	$closeOrders	= GETPOST('autocloseorders') != '' ? true : false;
+	$closeOrders	= GETPOST('autocloseorders') ? true : false;
 
 	// Security check
 	$fieldid = GETPOST('ref','alpha')?'facnumber':'rowid';
@@ -200,18 +207,19 @@ if (($action == 'create' || $action == 'add') && empty($mesgs))
 
 						while ($ii < $nn)
 						{
-							dol_include_once('/commande/class/commande.class.php');
-							$srcobject = new Commande($db);
+							include_once DOL_DOCUMENT_ROOT.'/commande/class/commande.class.php';
+							$objectsrc = new Commande($db);
 							dol_syslog("Try to find source object origin=".$object->origin." originid=".$object->origin_id." to add lines");
-							$result=$srcobject->fetch($orders_id[$ii]);
+							$result=$objectsrc->fetch($orders_id[$ii]);
 							if ($result > 0)
 							{
-								if($closeOrders) {
-									$srcobject->classer_facturee();
-									$srcobject->setStatut(3);
+								if ($closeOrders) 
+								{
+									$objectsrc->classifyBilled();
+									$objectsrc->setStatut(3);
 								}
-								$lines = $srcobject->lines;
-								if (empty($lines) && method_exists($srcobject,'fetch_lines'))  $lines = $srcobject->fetch_lines();
+								$lines = $objectsrc->lines;
+								if (empty($lines) && method_exists($objectsrc,'fetch_lines'))  $lines = $objectsrc->fetch_lines();
 								$fk_parent_line=0;
 								$num=count($lines);
 								for ($i=0;$i<$num;$i++)
@@ -261,7 +269,6 @@ if (($action == 'create' || $action == 'add') && empty($mesgs))
 											$fk_parent_line = 0;
 										}
 										$result = $object->addline(
-												$id,
 												$desc,
 												$lines[$i]->subprice,
 												$lines[$i]->qty,
@@ -278,7 +285,7 @@ if (($action == 'create' || $action == 'add') && empty($mesgs))
 												'HT',
 												0,
 												$product_type,
-												$lines[$i]->rang,
+												$ii,
 												$lines[$i]->special_code,
 												$object->origin,
 												$lines[$i]->rowid,
@@ -306,7 +313,7 @@ if (($action == 'create' || $action == 'add') && empty($mesgs))
 							}
 							else
 							{
-								$mesgs[]=$srcobject->error;
+								$mesgs[]=$objectsrc->error;
 								$error++;
 							}
 							$ii++;
@@ -366,7 +373,7 @@ if ($action == 'create' && empty($mesgs))
 		$remise_percent 	= $soc->remise_percent;
 	}
 	$remise_absolue 	= 0;
-	$dateinvoice		= empty($conf->global->MAIN_AUTOFILL_DATE)?-1:0;
+	$dateinvoice		= empty($conf->global->MAIN_AUTOFILL_DATE)?-1:'';
 
 	$absolute_discount=$soc->getAvailableDiscounts();
 	print '<form name="add" action="'.$_SERVER["PHP_SELF"].'" method="POST">';
@@ -406,10 +413,10 @@ if ($action == 'create' && empty($mesgs))
 
 	// Date invoice
 	print '<tr><td class="fieldrequired">'.$langs->trans('Date').'</td><td colspan="2">';
-	$html->select_date(0,'','','','',"add",1,1);
+	$html->select_date('','','','','',"add",1,1);
 	print '</td></tr>';
 	// Payment term
-	print '<tr><td nowrap>'.$langs->trans('PaymentConditionsShort').'</td><td colspan="2">';
+	print '<tr><td class="nowrap">'.$langs->trans('PaymentConditionsShort').'</td><td colspan="2">';
 	$html->select_conditions_paiements(isset($_POST['cond_reglement_id'])?$_POST['cond_reglement_id']:$cond_reglement_id,'cond_reglement_id');
 	print '</td></tr>';
 	// Payment mode
@@ -419,11 +426,30 @@ if ($action == 'create' && empty($mesgs))
 	// Project
 	if (! empty($conf->projet->enabled))
 	{
+		$formproject=new FormProjets($db);
+
 		$langs->load('projects');
 		print '<tr><td>'.$langs->trans('Project').'</td><td colspan="2">';
-		select_projects($soc->id, $projectid, 'projectid');
+		$formproject->select_projects($soc->id, $projectid, 'projectid');
 		print '</td></tr>';
 	}
+
+	include_once DOL_DOCUMENT_ROOT.'/commande/class/commande.class.php';
+	$objectsrc = new Commande($db);
+	$listoforders = array();
+	foreach ($selected as $sel)
+	{
+		$result=$objectsrc->fetch($sel);
+		if ($result > 0)
+		{
+			$listoforders[] = $objectsrc->ref;
+		}
+	}
+
+	// Other attributes
+	$parameters=array('objectsrc' => $objectsrc, 'idsrc' => $listoforders, 'colspan' => ' colspan="3"');
+	$reshook=$hookmanager->executeHooks('formObjectOptions',$parameters,$object,$action);    // Note that $action and $object may have been modified by hook
+
 	// Modele PDF
 	print '<tr><td>'.$langs->trans('Model').'</td>';
 	print '<td>';
@@ -438,18 +464,7 @@ if ($action == 'create' && empty($mesgs))
 	print '<td valign="top" colspan="2">';
 	print '<textarea name="note_public" wrap="soft" cols="70" rows="'.ROWS_3.'">';
 
-	dol_include_once('/commande/class/commande.class.php');
-	$srcobject = new Commande($db);
-	$listoforders = '';
-	foreach ($selected as $sel)
-	{
-		$result=$srcobject->fetch($sel);
-		if ($result > 0)
-		{
-			$listoforders .= ($listoforders?', ':'').$srcobject->ref;
-		}
-	}
-	print $langs->trans("Orders").": ".$listoforders;
+	print $langs->trans("Orders").": ".implode(', ', $listoforders);
 
 	print '</textarea></td></tr>';
 	// Private note
@@ -588,7 +603,7 @@ if (($action != 'create' && $action != 'add') || ! empty($mesgs))
 
 		//SEARCH BUTTON
 		print '</td><td align="right" class="liste_titre">';
-		print '<input type="image" class="liste_titre" name="button_search" src="'.DOL_URL_ROOT.'/theme/'.$conf->theme.'/img/search.png"  value="'.dol_escape_htmltag($langs->trans("Search")).'" title="'.dol_escape_htmltag($langs->trans("Search")).'">';
+		print '<input type="image" class="liste_titre" name="button_search" src="'.img_picto($langs->trans("Search"),'search.png','','',1).'"  value="'.dol_escape_htmltag($langs->trans("Search")).'" title="'.dol_escape_htmltag($langs->trans("Search")).'">';
 
 		//ALL/NONE
 		print '<td class="liste_titre" align="center">';
@@ -607,21 +622,21 @@ if (($action != 'create' && $action != 'add') || ! empty($mesgs))
 			$objp = $db->fetch_object($resql);
 			$var=!$var;
 			print '<tr '.$bc[$var].'>';
-			print '<td nowrap="nowrap">';
+			print '<td class="nowrap">';
 
 			$generic_commande->id=$objp->rowid;
 			$generic_commande->ref=$objp->ref;
 
 			print '<table class="nobordernopadding"><tr class="nocellnopadd">';
-			print '<td class="nobordernopadding" nowrap="nowrap">';
+			print '<td class="nobordernopadding nowrap">';
 			print $generic_commande->getNomUrl(1,$objp->fk_statut);
 			print '</td>';
 
-			print '<td width="20" class="nobordernopadding" nowrap="nowrap">';
+			print '<td width="20" class="nobordernopadding nowrap">';
 			if (($objp->fk_statut > 0) && ($objp->fk_statut < 3) && $db->jdate($objp->date_valid) < ($now - $conf->commande->client->warning_delay)) print img_picto($langs->trans("Late"),"warning");
 			print '</td>';
 
-			print '<td width="16" align="right" class="nobordernopadding">';
+			print '<td width="16" align="right" class="nobordernopadding hideonsmartphone">';
 			$filename=dol_sanitizeFileName($objp->ref);
 			$filedir=$conf->commande->dir_output . '/' . dol_sanitizeFileName($objp->ref);
 			$urlsource=$_SERVER['PHP_SELF'].'?id='.$objp->rowid;
@@ -642,7 +657,7 @@ if (($action != 'create' && $action != 'add') || ! empty($mesgs))
 			print '</td>';
 
 			// Statut
-			print '<td align="right" nowrap="nowrap">'.$generic_commande->LibStatut($objp->fk_statut,$objp->facturee,5).'</td>';
+			print '<td align="right" class="nowrap">'.$generic_commande->LibStatut($objp->fk_statut,$objp->facturee,5).'</td>';
 
 			// Checkbox
 			print '<td align="center">';
@@ -682,4 +697,4 @@ dol_htmloutput_mesg($mesg,$mesgs);
 
 llxFooter();
 $db->close();
-?>
\ No newline at end of file
+?>
