@@ -27,7 +27,7 @@
  *		\brief      List of details of bank transactions for an account
  */
 
-require 'pre.inc.php';	// We use pre.inc.php to have a dynamic menu
+require('../../main.inc.php');
 require_once DOL_DOCUMENT_ROOT.'/core/lib/bank.lib.php';
 require_once DOL_DOCUMENT_ROOT.'/societe/class/societe.class.php';
 require_once DOL_DOCUMENT_ROOT.'/adherents/class/adherent.class.php';
@@ -35,7 +35,10 @@ require_once DOL_DOCUMENT_ROOT.'/compta/sociales/class/chargesociales.class.php'
 require_once DOL_DOCUMENT_ROOT.'/compta/paiement/class/paiement.class.php';
 require_once DOL_DOCUMENT_ROOT.'/compta/tva/class/tva.class.php';
 require_once DOL_DOCUMENT_ROOT.'/fourn/class/paiementfourn.class.php';
+require_once DOL_DOCUMENT_ROOT.'/compta/bank/class/account.class.php';
 
+$langs->load("banks");
+$langs->load("categories");
 $langs->load("bills");
 
 $id = (GETPOST('id','int') ? GETPOST('id','int') : GETPOST('account','int'));
@@ -47,18 +50,22 @@ $confirm=GETPOST('confirm','alpha');
 $fieldvalue = (! empty($id) ? $id : (! empty($ref) ? $ref :''));
 $fieldtype = (! empty($ref) ? 'ref' :'rowid');
 if ($user->societe_id) $socid=$user->societe_id;
-$result=restrictedArea($user,'banque',$fieldvalue,'bank_account','','',$fieldtype);
+$result=restrictedArea($user,'banque',$fieldvalue,'bank_account&bank_account','','',$fieldtype);
 
 $paiementtype=GETPOST('paiementtype','alpha',3);
 $req_nb=GETPOST("req_nb",'',3);
 $thirdparty=GETPOST("thirdparty",'',3);
+$req_desc=GETPOST("req_desc",'',3);
+$req_debit=GETPOST("req_debit",'',3);
+$req_credit=GETPOST("req_credit",'',3);
+
 $vline=GETPOST("vline");
 $page=GETPOST('page','int');
 $negpage=GETPOST('negpage','int');
 if ($negpage)
 {
-    $page=$_GET["nbpage"] - $negpage;
-    if ($page > $_GET["nbpage"]) $page = $_GET["nbpage"];
+    $page=GETPOST("nbpage") - $negpage;
+    if ($page > GETPOST("nbpage")) $page = GETPOST("nbpage");
 }
 
 $mesg='';
@@ -97,6 +104,7 @@ if ($action == 'add' && $id && ! isset($_POST["cancel"]) && $user->rights->banqu
 		$insertid = $object->addline($dateop, $operation, $label, $amount, $num_chq, $cat1, $user);
 		if ($insertid > 0)
 		{
+			setEventMessage($langs->trans("RecordSaved"));
 			header("Location: ".$_SERVER['PHP_SELF']."?id=".$id."&action=addline");
 			exit;
 		}
@@ -113,7 +121,7 @@ if ($action == 'add' && $id && ! isset($_POST["cancel"]) && $user->rights->banqu
 if ($action == 'confirm_delete' && $confirm == 'yes' && $user->rights->banque->modifier)
 {
 	$accline=new AccountLine($db);
-	$accline->fetch($_GET["rowid"]);
+	$result=$accline->fetch(GETPOST("rowid"));
 	$result=$accline->delete();
 }
 
@@ -130,6 +138,8 @@ $memberstatic=new Adherent($db);
 $paymentstatic=new Paiement($db);
 $paymentsupplierstatic=new PaiementFourn($db);
 $paymentvatstatic=new TVA($db);
+$bankstatic=new Account($db);
+$banklinestatic=new AccountLine($db);
 
 $form = new Form($db);
 
@@ -181,22 +191,22 @@ if ($id > 0 || ! empty($ref))
 		$param.='&amp;req_nb='.urlencode($req_nb);
 		$mode_search = 1;
 	}
-	if (GETPOST("req_desc"))
+	if ($req_desc)
 	{
-		$sql_rech.= " AND b.label LIKE '%".$db->escape(GETPOST("req_desc"))."%'";
-		$param.='&amp;req_desc='.urlencode(GETPOST("req_desc"));
+		$sql_rech.= " AND b.label LIKE '%".$db->escape($req_desc)."%'";
+		$param.='&amp;req_desc='.urlencode($req_desc);
 		$mode_search = 1;
 	}
-	if (GETPOST("req_debit"))
+	if ($req_debit != '')
 	{
-		$sql_rech.=" AND b.amount = -".price2num(GETPOST("req_debit"));
-		$param.='&amp;req_debit='.urlencode(GETPOST("req_debit"));
+		$sql_rech.=" AND b.amount = -".price2num($req_debit);
+		$param.='&amp;req_debit='.urlencode($req_debit);
 		$mode_search = 1;
 	}
-	if (GETPOST("req_credit"))
+	if ($req_credit != '')
 	{
-		$sql_rech.=" AND b.amount = ".price2num(GETPOST("req_credit"));
-		$param.='&amp;req_credit='.urlencode(GETPOST("req_credit"));
+		$sql_rech.=" AND b.amount = ".price2num($req_credit);
+		$param.='&amp;req_credit='.urlencode($req_credit);
 		$mode_search = 1;
 	}
 	if ($thirdparty)
@@ -260,6 +270,7 @@ if ($id > 0 || ! empty($ref))
 	$head=bank_prepare_head($object);
 	dol_fiche_head($head,'journal',$langs->trans("FinancialAccount"),0,'account');
 
+
 	print '<table class="border" width="100%">';
 
 	$linkback = '<a href="'.DOL_URL_ROOT.'/compta/bank/index.php">'.$langs->trans("BackToList").'</a>';
@@ -283,55 +294,54 @@ if ($id > 0 || ! empty($ref))
 	/**
 	 * Search form
 	 */
-	$param.='&amp;account='.$object->id;
-
-	// Define transaction list navigation string
-	$navig = '<form action="'.$_SERVER["PHP_SELF"].'" name="newpage" method="GET">';
-	//print 'nbpage='.$totalPages.' viewline='.$viewline.' limitsql='.$limitsql;
-	if ($limitsql > $viewline) $navig.='<a href="account.php?'.$param.'&amp;page='.($page+1).'">'.img_previous().'</a>';
-	$navig.= $langs->trans("Page")." "; // ' Page ';
-	$navig.='<input type="text" name="negpage" size="1" class="flat" value="'.($totalPages-$page).'">';
-	$navig.='<input type="hidden" name="paiementtype" value="'.$paiementtype.'">';
-	$navig.='<input type="hidden" name="req_nb"     value="'.$req_nb.'">';
-	$navig.='<input type="hidden" name="req_desc"   value="'.GETPOST("req_desc").'">';
-	$navig.='<input type="hidden" name="req_debit"  value="'.GETPOST("req_debit").'">';
-	$navig.='<input type="hidden" name="req_credit" value="'.GETPOST("req_credit").'">';
-	$navig.='<input type="hidden" name="thirdparty" value="'.$thirdparty.'">';
-	$navig.='<input type="hidden" name="nbpage"  value="'.$totalPages.'">';
-	$navig.='<input type="hidden" name="id" value="'.$object->id.'">';
-	$navig.='/'.$totalPages.' ';
-	if ($total_lines > $limitsql )
-	{
-		$navig.= '<a href="'.$_SERVER["PHP_SELF"].'?'.$param.'&amp;page='.($page-1).'">'.img_next().'</a>';
-	}
-	$navig.='</form>';
-	//var_dump($navig);
+	$param.='&amp;account='.$object->id.'&amp;vline='.$vline;
 
 	// Confirmation delete
 	if ($action == 'delete')
 	{
 		$text=$langs->trans('ConfirmDeleteTransaction');
-		$ret=$form->form_confirm($_SERVER['PHP_SELF'].'?id='.$object->id.'&amp;rowid='.$_GET["rowid"],$langs->trans('DeleteTransaction'),$text,'confirm_delete');
-		if ($ret == 'html') print '<br>';
+		print $form->formconfirm($_SERVER['PHP_SELF'].'?id='.$object->id.'&amp;rowid='.GETPOST("rowid"),$langs->trans('DeleteTransaction'),$text,'confirm_delete');
+
 	}
 
+	// Define transaction list navigation string
+	print '<form action="'.$_SERVER["PHP_SELF"].'" name="newpage" method="POST">';
+	print '<input type="hidden" name="token"        value="'.$_SESSION['newtoken'].'">';
+	print '<input type="hidden" name="action"       value="add">';
+	print '<input type="hidden" name="vline"        value="'.$vline.'">';
+	print '<input type="hidden" name="paiementtype" value="'.$paiementtype.'">';
+	print '<input type="hidden" name="req_nb"       value="'.$req_nb.'">';
+	print '<input type="hidden" name="req_desc"     value="'.$req_desc.'">';
+	print '<input type="hidden" name="req_debit"    value="'.$req_debit.'">';
+	print '<input type="hidden" name="req_credit"   value="'.$req_credit.'">';
+	print '<input type="hidden" name="thirdparty"   value="'.$thirdparty.'">';
+	print '<input type="hidden" name="nbpage"       value="'.$totalPages.'">';
+	print '<input type="hidden" name="id"           value="'.$object->id.'">';
+
+	$navig ='<div data-role="fieldcontain">';
+	if ($limitsql > $viewline) $navig.='<a href="account.php?'.$param.'&amp;page='.($page+1).'">'.img_previous().'</a>';
+	$navig.= '<label for="negpage">'.$langs->trans("Page")."</label> "; // ' Page ';
+	$navig.='<input type="text" name="negpage" id="negpage" size="1" class="flat" value="'.($totalPages-$page).'">';
+	$navig.='/'.$totalPages.' ';
+	if ($total_lines > $limitsql )
+	{
+		$navig.= '<a href="'.$_SERVER["PHP_SELF"].'?'.$param.'&amp;page='.($page-1).'">'.img_next().'</a>';
+	}
+	$navig.='</div>';
+
+	//var_dump($navig);
+
 	print '<table class="notopnoleftnoright" width="100%">';
 
 	// Show title
 	if ($action != 'addline' && $action != 'delete')
 	{
-		print '<tr><td colspan="9" align="right">'.$navig.'</td></tr>';
+		print '<tr><td colspan="10" align="right">'.$navig.'</td></tr>';
 	}
 
 	// Form to add a transaction with no invoice
 	if ($user->rights->banque->modifier && $action == 'addline')
 	{
-		print '<form method="POST" action="'.$_SERVER["PHP_SELF"].'">';
-		print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
-		print '<input type="hidden" name="action" value="add">';
-		print '<input type="hidden" name="vline" value="'.$vline.'">';
-		print '<input type="hidden" name="id" value="'.$object->id.'">';
-
 		print '<tr>';
 		print '<td align="left" colspan="10"><b>'.$langs->trans("AddBankRecordLong").'</b></td>';
 		print '</tr>';
@@ -348,10 +358,10 @@ if ($id > 0 || ! empty($ref))
 		print '</tr>';
 
 		print '<tr '.$bc[false].'>';
-		print '<td nowrap="nowrap" colspan="2">';
+		print '<td class="nowrap" colspan="2">';
 		$form->select_date($dateop,'op',0,0,0,'transaction');
 		print '</td>';
-		print '<td nowrap="nowrap">';
+		print '<td class="nowrap">';
 		$form->select_types_paiements((GETPOST('operation')?GETPOST('operation'):($object->courant == 2 ? 'LIQ' : '')),'operation','1,2',2,1);
 		print '</td><td>';
 		print '<input name="num_chq" class="flat" type="text" size="4" value="'.GETPOST("num_chq").'"></td>';
@@ -370,7 +380,7 @@ if ($id > 0 || ! empty($ref))
 		print '</td></tr>';
 		print "</form>";
 
-		print '<tr class="noborder"><td colspan="8">&nbsp;</td></tr>'."\n";
+		print '<tr class="noborder"><td colspan="10">&nbsp;</td></tr>'."\n";
 	}
 
 	/*
@@ -407,14 +417,14 @@ if ($id > 0 || ! empty($ref))
 	print $form->select_types_paiements($paiementtype,'paiementtype',$filtertype,2,1,1,8);
 	print '</td>';
 	print '<td><input type="text" class="flat" name="req_nb" value="'.$req_nb.'" size="2"></td>';
-	print '<td><input type="text" class="flat" name="req_desc" value="'.GETPOST("req_desc").'" size="24"></td>';
+	print '<td><input type="text" class="flat" name="req_desc" value="'.$req_desc.'" size="24"></td>';
 	print '<td><input type="text" class="flat" name="thirdparty" value="'.$thirdparty.'" size="14"></td>';
-	print '<td align="right"><input type="text" class="flat" name="req_debit" value="'.GETPOST("req_debit").'" size="4"></td>';
-	print '<td align="right"><input type="text" class="flat" name="req_credit" value="'.GETPOST("req_credit").'" size="4"></td>';
+	print '<td align="right"><input type="text" class="flat" name="req_debit" value="'.$req_debit.'" size="4"></td>';
+	print '<td align="right"><input type="text" class="flat" name="req_credit" value="'.$req_credit.'" size="4"></td>';
 	print '<td align="center">&nbsp;</td>';
-	print '<td align="center" width="40"><input type="image" class="liste_titre" src="'.DOL_URL_ROOT.'/theme/'.$conf->theme.'/img/search.png" value="'.dol_escape_htmltag($langs->trans("Search")).'" title="'.dol_escape_htmltag($langs->trans("Search")).'"></td>';
+	print '<td align="center" width="40"><input type="image" class="liste_titre" src="'.img_picto($langs->trans("Search"),'search.png','','',1).'" value="'.dol_escape_htmltag($langs->trans("Search")).'" title="'.dol_escape_htmltag($langs->trans("Search")).'"></td>';
 	print "</tr>\n";
-	print "</form>\n";
+
 
 	/*
 	 * Another solution
@@ -423,7 +433,8 @@ if ($id > 0 || ! empty($ref))
      */
 
 	$sql = "SELECT b.rowid, b.dateo as do, b.datev as dv,";
-	$sql.= " b.amount, b.label, b.rappro, b.num_releve, b.num_chq, b.fk_type";
+	$sql.= " b.amount, b.label, b.rappro, b.num_releve, b.num_chq, b.fk_type, b.fk_bordereau,";
+	$sql.= " ba.rowid as bankid, ba.ref as bankref, ba.label as banklabel";
 	if ($mode_search)
 	{
 		$sql.= ", s.rowid as socid, s.nom as thirdparty";
@@ -476,7 +487,7 @@ if ($id > 0 || ! empty($ref))
 		$var=true;
 
 		$num = $db->num_rows($result);
-		$i = 0; $total = 0; $sep = 0;
+		$i = 0; $total = 0; $sep = -1;
 
 		while ($i < $num)
 		{
@@ -489,34 +500,41 @@ if ($id > 0 || ! empty($ref))
 				// Is it a transaction in future ?
 				$dos=dol_print_date($db->jdate($objp->do),'%Y%m%d');
 				//print "dos=".$dos." nows=".$nows;
-				if ($dos > $nows && !$sep)		// Yes, we show a subtotal
+				if ($dos < $nows) $sep=0;		// 0 means there was at least one line before current date
+				if ($dos > $nows && ! $sep)		// We have found a line in future and we already found on line before current date
 				{
 					$sep = 1 ;
 					print '<tr class="liste_total"><td colspan="8">';
 					print $langs->trans("CurrentBalance");
 					print '</td>';
-					print '<td align="right" nowrap><b>'.price($total - $objp->amount).'</b></td>';
+					print '<td align="right" class="nowrap"><b>'.price($total - $objp->amount).'</b></td>';
 					print "<td>&nbsp;</td>";
 					print '</tr>';
 				}
 
 				print '<tr '.$bc[$var].'>';
 
-				print '<td nowrap="nowrap">'.dol_print_date($db->jdate($objp->do),"day")."</td>\n";
+				print '<td class="nowrap">'.dol_print_date($db->jdate($objp->do),"day")."</td>\n";
 
-				print '<td nowrap="nowrap">'.dol_print_date($db->jdate($objp->dv),"day");
+				print '<td class="nowrap">'.dol_print_date($db->jdate($objp->dv),"day");
 				print "</td>\n";
 
 				// Payment type
-				print "<td nowrap>";
+				print '<td class="nowrap">';
 				$label=($langs->trans("PaymentTypeShort".$objp->fk_type)!="PaymentTypeShort".$objp->fk_type)?$langs->trans("PaymentTypeShort".$objp->fk_type):$objp->fk_type;
 
 				if ($objp->fk_type == 'SOLD') $label='&nbsp;';
+				if ($objp->fk_type == 'CHQ' && $objp->fk_bordereau > 0) {
+					dol_include_once('/compta/paiement/cheque/class/remisecheque.class.php');
+					$bordereaustatic = new RemiseCheque($db);
+					$bordereaustatic->id = $objp->fk_bordereau;
+					$label .= ' '.$bordereaustatic->getNomUrl(2);
+				}
 				print $label;
 				print "</td>\n";
 
 				// Num
-				print '<td nowrap>'.($objp->num_chq?$objp->num_chq:"")."</td>\n";
+				print '<td class="nowrap">'.($objp->num_chq?$objp->num_chq:"")."</td>\n";
 
 				// Description
 				print '<td>';
@@ -537,41 +555,68 @@ if ($id > 0 || ! empty($ref))
 					if ($links[$key]['type']=='payment')
 					{
 						$paymentstatic->id=$links[$key]['url_id'];
+						$paymentstatic->ref=$links[$key]['url_id'];
 						print ' '.$paymentstatic->getNomUrl(2);
 					}
-					else if ($links[$key]['type']=='payment_supplier')
+					elseif ($links[$key]['type']=='payment_supplier')
 					{
 						$paymentsupplierstatic->id=$links[$key]['url_id'];
 						$paymentsupplierstatic->ref=$links[$key]['url_id'];
 						print ' '.$paymentsupplierstatic->getNomUrl(2);
 					}
-					else if ($links[$key]['type']=='company')
+					elseif ($links[$key]['type']=='payment_sc')
 					{
-
-					}
-					else if ($links[$key]['type']=='payment_sc')
-					{
-						//print ' - ';
 						print '<a href="'.DOL_URL_ROOT.'/compta/payment_sc/fiche.php?id='.$links[$key]['url_id'].'">';
 						print ' '.img_object($langs->trans('ShowPayment'),'payment').' ';
 						//print $langs->trans("SocialContributionPayment");
 						print '</a>';
 					}
-					else if ($links[$key]['type']=='payment_vat')
+					elseif ($links[$key]['type']=='payment_vat')
 					{
 						$paymentvatstatic->id=$links[$key]['url_id'];
 						$paymentvatstatic->ref=$links[$key]['url_id'];
 						print ' '.$paymentvatstatic->getNomUrl(2);
 					}
-					else if ($links[$key]['type']=='banktransfert')
+					elseif ($links[$key]['type']=='banktransfert')
 					{
-						// Do not show this link (avoid confusion). Can already be accessed from transaction detail
+						// Do not show link to transfer since there is no transfer card (avoid confusion). Can already be accessed from transaction detail.
+						if ($objp->amount > 0)
+						{
+							$banklinestatic->fetch($links[$key]['url_id']);
+							$bankstatic->id=$banklinestatic->fk_account;
+							$bankstatic->label=$banklinestatic->bank_account_label;
+							print ' ('.$langs->trans("TransferFrom").' ';
+							print $bankstatic->getNomUrl(1,'transactions');
+							print ' '.$langs->trans("toward").' ';
+							$bankstatic->id=$objp->bankid;
+							$bankstatic->label=$objp->bankref;
+							print $bankstatic->getNomUrl(1,'');
+							print ')';
+						}
+						else
+						{
+							$bankstatic->id=$objp->bankid;
+							$bankstatic->label=$objp->bankref;
+							print ' ('.$langs->trans("TransferFrom").' ';
+							print $bankstatic->getNomUrl(1,'');
+							print ' '.$langs->trans("toward").' ';
+							$banklinestatic->fetch($links[$key]['url_id']);
+							$bankstatic->id=$banklinestatic->fk_account;
+							$bankstatic->label=$banklinestatic->bank_account_label;
+							print $bankstatic->getNomUrl(1,'transactions');
+							print ')';
+						}
+						//var_dump($links);
 					}
-					else if ($links[$key]['type']=='member')
+					elseif ($links[$key]['type']=='company')
 					{
 
 					}
-					else if ($links[$key]['type']=='sc')
+					elseif ($links[$key]['type']=='member')
+					{
+
+					}
+					elseif ($links[$key]['type']=='sc')
 					{
 
 					}
@@ -633,15 +678,15 @@ if ($id > 0 || ! empty($ref))
 				// Amount
 				if ($objp->amount < 0)
 				{
-					print '<td align="right" nowrap="nowrap">'.price($objp->amount * -1).'</td><td>&nbsp;</td>'."\n";
+					print '<td align="right" class="nowrap">'.price($objp->amount * -1).'</td><td>&nbsp;</td>'."\n";
 				}
 				else
 				{
-					print '<td>&nbsp;</td><td align="right" nowrap="nowrap">&nbsp;'.price($objp->amount).'</td>'."\n";
+					print '<td>&nbsp;</td><td align="right" class="nowrap">&nbsp;'.price($objp->amount).'</td>'."\n";
 				}
 
 				// Balance
-				if ($action != 'search')
+				if (! $mode_search)
 				{
 					if ($total >= 0)
 					{
@@ -710,10 +755,10 @@ if ($id > 0 || ! empty($ref))
 		if ($page == 0 && ! $mode_search)
 		{
 			print '<tr class="liste_total"><td align="left" colspan="8">';
-			if ($sep) print '&nbsp;';
+			if ($sep > 0) print '&nbsp;';	// If we had at least one line in future
 			else print $langs->trans("CurrentBalance");
 			print '</td>';
-			print '<td align="right" nowrap>'.price($total).'</td>';
+			print '<td align="right" nowrap><b>'.price($total).'</b></td>';
 			print '<td>&nbsp;</td>';
 			print '</tr>';
 		}
@@ -726,7 +771,10 @@ if ($id > 0 || ! empty($ref))
 
 	print "</table>";
 
-	print "\n</div>\n";
+	print "</form>\n";
+
+	dol_fiche_end();
+
 
 	/*
 	 * Boutons actions
@@ -740,7 +788,7 @@ if ($id > 0 || ! empty($ref))
 		{
 			if ($user->rights->banque->consolidate)
 			{
-				print '<a class="butAction" href="'.DOL_URL_ROOT.'/compta/bank/rappro.php?account='.$object->id.'">'.$langs->trans("Conciliate").'</a>';
+				print '<a class="butAction" href="'.DOL_URL_ROOT.'/compta/bank/rappro.php?account='.$object->id.($vline?'&amp;vline='.$vline:'').'">'.$langs->trans("Conciliate").'</a>';
 			}
 			else
 			{
@@ -750,13 +798,20 @@ if ($id > 0 || ! empty($ref))
 
 		if ($action != 'addline')
 		{
-			if ($user->rights->banque->modifier)
+			if (empty($conf->global->BANK_DISABLE_DIRECT_INPUT))
 			{
-				print '<a class="butAction" href="'.$_SERVER["PHP_SELF"].'?action=addline&amp;id='.$object->id.'&amp;page='.$page.'">'.$langs->trans("AddBankRecord").'</a>';
+				if ($user->rights->banque->modifier)
+				{
+					print '<a class="butAction" href="'.$_SERVER["PHP_SELF"].'?action=addline&amp;id='.$object->id.'&amp;page='.$page.($vline?'&amp;vline='.$vline:'').'">'.$langs->trans("AddBankRecord").'</a>';
+				}
+				else
+				{
+					print '<a class="butActionRefused" title="'.$langs->trans("NotEnoughPermissions").'" href="#">'.$langs->trans("AddBankRecord").'</a>';
+				}
 			}
 			else
 			{
-				print '<a class="butActionRefused" title="'.$langs->trans("NotEnoughPermissions").'" href="#">'.$langs->trans("AddBankRecord").'</a>';
+				print '<a class="butActionRefused" title="'.$langs->trans("FeatureDisabled").'" href="#">'.$langs->trans("AddBankRecord").'</a>';
 			}
 		}
 
@@ -773,4 +828,4 @@ else
 llxFooter();
 
 $db->close();
-?>
\ No newline at end of file
+?>
