@@ -2,6 +2,7 @@
 /* Copyright (C) 2002-2004 Rodolphe Quiedeville  <rodolphe@quiedeville.org>
  * Copyright (C) 2004-2010 Laurent Destailleur   <eldy@users.sourceforge.net>
  * Copyright (C)      2005 Marc Barilley / Ocebo <marc@ocebo.com>
+ * Copyright (C) 2014      Marcos García <marcosgdf@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -209,7 +210,15 @@ class Paiement extends CommonObject
                                 }
                             }
 
-                            if ($invoice->type != 0 && $invoice->type != 1 && $invoice->type != 2) dol_syslog("Invoice ".$facid." is not a standard, nor replacement invoice, nor credit note. We do nothing more.");
+                            //Invoice types that are eligible for changing status to paid
+							$affected_types = array(
+								0,
+								1,
+								2,
+								3
+							);
+
+                            if (!in_array($invoice->type, $affected_types)) dol_syslog("Invoice ".$facid." is not a standard, nor replacement invoice, nor credit note, nor deposit invoice. We do nothing more.");
                             else if ($remaintopay) dol_syslog("Remain to pay for invoice ".$facid." not null. We do nothing more.");
                             else if ($mustwait) dol_syslog("There is ".$mustwait." differed payment to process, we do nothing more.");
                             else $result=$invoice->set_paid($user,'','');
@@ -366,7 +375,7 @@ class Paiement extends CommonObject
 
 
     /**
-     *      A record into bank for payment with links between this bank record and invoices of payment.
+     *      Add a record into bank for payment with links between this bank record and invoices of payment.
      *      All payment properties (this->amount, this->amounts, ...) must have been set first like after a call to create().
      *
      *      @param	User	$user               Object of user making payment
@@ -384,16 +393,26 @@ class Paiement extends CommonObject
 
         $error=0;
         $bank_line_id=0;
-        $this->fk_account=$accountid;
 
         if (! empty($conf->banque->enabled))
         {
-            require_once DOL_DOCUMENT_ROOT.'/compta/bank/class/account.class.php';
+        	if ($accountid <= 0)
+        	{
+        		$this->error='Bad value for parameter accountid';
+        		dol_syslog(get_class($this).'::addPaymentToBank '.$this->error, LOG_ERR);
+        		return -1;
+        	}
+
+        	$this->db->begin();
+
+        	$this->fk_account=$accountid;
+
+        	require_once DOL_DOCUMENT_ROOT.'/compta/bank/class/account.class.php';
 
             dol_syslog("$user->id,$mode,$label,$this->fk_account,$emetteur_nom,$emetteur_banque");
 
             $acc = new Account($this->db);
-            $acc->fetch($this->fk_account);
+            $result=$acc->fetch($this->fk_account);
 
             $totalamount=$this->amount;
             if (empty($totalamount)) $totalamount=$this->total; // For backward compatibility
@@ -442,7 +461,7 @@ class Paiement extends CommonObject
                 }
 
                 // Add link 'company' in bank_url between invoice and bank transaction (for each invoice concerned by payment)
-                if (! $error)
+                if (! $error  && $label != '(WithdrawalPayment)')
                 {
                     $linkaddedforthirdparty=array();
                     foreach ($this->amounts as $key => $value)  // We should have always same third party but we loop in case of.
@@ -497,10 +516,19 @@ class Paiement extends CommonObject
 				}
             }
             else
-            {
+			{
                 $this->error=$acc->error;
                 $error++;
             }
+
+            if (! $error)
+            {
+            	$this->db->commit();
+            }
+            else
+			{
+            	$this->db->rollback();
+            }
         }
 
         if (! $error)
