@@ -153,10 +153,10 @@ class FormOther
 
 
     /**
-     *    Retourne la liste des ecotaxes avec tooltip sur le libelle
+     *    Return list of ecotaxes with label
      *
-     *    @param	string	$selected    code ecotaxes pre-selectionne
-     *    @param    string	$htmlname    nom de la liste deroulante
+     *    @param	string	$selected   Preselected ecotaxes
+     *    @param    string	$htmlname	Name of combo list
      *    @return	void
      */
     function select_ecotaxes($selected='',$htmlname='ecotaxe_id')
@@ -169,6 +169,7 @@ class FormOther
         $sql.= " WHERE e.active = 1 AND e.fk_pays = p.rowid";
         $sql.= " ORDER BY pays, e.organization ASC, e.code ASC";
 
+    	dol_syslog(get_class($this).'::select_ecotaxes sql='.$sql);
         $resql=$this->db->query($sql);
         if ($resql)
         {
@@ -206,6 +207,64 @@ class FormOther
         }
     }
 
+
+    /**
+     *    Return list of revenue stamp for country
+     *
+     *    @param	string	$selected   	Value of preselected revenue stamp
+     *    @param    string	$htmlname   	Name of combo list
+     *    @param    string	$country_code   Country Code
+     *    @return	string					HTML select list
+     */
+    function select_revenue_stamp($selected='',$htmlname='revenuestamp',$country_code='')
+    {
+    	global $langs;
+
+    	$out='';
+
+    	$sql = "SELECT r.taux";
+    	$sql.= " FROM ".MAIN_DB_PREFIX."c_revenuestamp as r,".MAIN_DB_PREFIX."c_pays as p";
+    	$sql.= " WHERE r.active = 1 AND r.fk_pays = p.rowid";
+    	$sql.= " AND p.code = '".$country_code."'";
+
+    	dol_syslog(get_class($this).'::select_revenue_stamp sql='.$sql);
+    	$resql=$this->db->query($sql);
+    	if ($resql)
+    	{
+    		$out.='<select class="flat" name="'.$htmlname.'">';
+    		$num = $this->db->num_rows($resql);
+    		$i = 0;
+    		$out.='<option value="0">&nbsp;</option>'."\n";
+    		if ($num)
+    		{
+    			while ($i < $num)
+    			{
+    				$obj = $this->db->fetch_object($resql);
+    				if (($selected && $selected == $obj->taux) || $num == 1)
+    				{
+    					$out.='<option value="'.$obj->taux.'" selected="selected">';
+    				}
+    				else
+    				{
+    					$out.='<option value="'.$obj->taux.'">';
+    					//print '<option onmouseover="showtip(\''.$obj->libelle.'\')" onMouseout="hidetip()" value="'.$obj->rowid.'">';
+    				}
+    				$out.=$obj->taux;
+    				$out.='</option>';
+    				$i++;
+    			}
+    		}
+    		$out.='</select>';
+    		return $out;
+    	}
+    	else
+    	{
+    		dol_print_error($this->db);
+    		return '';
+    	}
+    }
+
+
     /**
      *    Return a HTML select list to select a percent
      *
@@ -288,31 +347,34 @@ class FormOther
      *  @param	string	$selected     	Preselected value
      *  @param  string	$htmlname      	Name of combo list (example: 'search_sale')
      *  @param  User	$user           Object user
+     *  @param	int		$showstatus		0=show user status only if status is disabled, 1=always show user status into label, -1=never show user status
      *  @return string					Html combo list code
      */
-    function select_salesrepresentatives($selected,$htmlname,$user)
+    function select_salesrepresentatives($selected,$htmlname,$user,$showstatus=0)
     {
-        global $conf;
+        global $conf,$langs;
+        $langs->load('users');
 
         // Select each sales and print them in a select input
         $moreforfilter ='<select class="flat" name="'.$htmlname.'">';
         $moreforfilter.='<option value="">&nbsp;</option>';
 
         // Get list of users allowed to be viewed
-        $sql_usr = "SELECT u.rowid, u.name as name, u.firstname, u.login";
+        $sql_usr = "SELECT u.rowid, u.lastname, u.firstname, u.statut, u.login";
         $sql_usr.= " FROM ".MAIN_DB_PREFIX."user as u";
         $sql_usr.= " WHERE u.entity IN (0,".$conf->entity.")";
-        if (empty($user->rights->user->user->lire)) $sql_usr.=" AND u.fk_societe = ".($user->societe_id?$user->societe_id:0);
-        // Add existing sales representatives of company
+        if (empty($user->rights->user->user->lire)) $sql_usr.=" AND u.rowid = ".$user->id;
+        if (! empty($user->societe_id)) $sql_usr.=" AND u.fk_societe = ".$user->societe_id;
+        // Add existing sales representatives of thirdparty of external user
         if (empty($user->rights->user->user->lire) && $user->societe_id)
         {
             $sql_usr.=" UNION ";
-            $sql_usr.= "SELECT u2.rowid, u2.name as name, u2.firstname, u2.login";
+            $sql_usr.= "SELECT u2.rowid, u2.lastname, u2.firstname, u2.statut, u2.login";
             $sql_usr.= " FROM ".MAIN_DB_PREFIX."user as u2, ".MAIN_DB_PREFIX."societe_commerciaux as sc";
             $sql_usr.= " WHERE u2.entity IN (0,".$conf->entity.")";
             $sql_usr.= " AND u2.rowid = sc.fk_user AND sc.fk_soc=".$user->societe_id;
         }
-        $sql_usr.= " ORDER BY name ASC";
+        $sql_usr.= " ORDER BY lastname ASC";
         //print $sql_usr;exit;
 
         $resql_usr = $this->db->query($sql_usr);
@@ -325,7 +387,28 @@ class FormOther
                 if ($obj_usr->rowid == $selected) $moreforfilter.=' selected="selected"';
 
                 $moreforfilter.='>';
-                $moreforfilter.=$obj_usr->firstname." ".$obj_usr->name." (".$obj_usr->login.')';
+                $moreforfilter.=dolGetFirstLastname($obj_usr->firstname,$obj_usr->lastname);
+                // Complete name with more info
+                $moreinfo=0;
+                if (! empty($conf->global->MAIN_SHOW_LOGIN))
+                {
+                	$out.=($moreinfo?' - ':' (').$obj->login;
+                	$moreinfo++;
+                }
+                if ($showstatus >= 0)
+                {
+					if ($obj_usr->statut == 1 && $showstatus == 1)
+					{
+						$moreforfilter.=($moreinfo?' - ':' (').$langs->trans('Enabled');
+	                	$moreinfo++;
+					}
+					if ($obj_usr->statut == 0)
+					{
+						$moreforfilter.=($moreinfo?' - ':' (').$langs->trans('Disabled');
+                		$moreinfo++;
+					}
+				}
+				$moreforfilter.=($moreinfo?')':'');
                 $moreforfilter.='</option>';
             }
             $this->db->free($resql_usr);
@@ -342,16 +425,17 @@ class FormOther
     /**
      *	Return list of project and tasks
      *
-     *	@param  int		$selectedtask   	Pre-selected task
-     *  @param  int		$projectid       Project id
-     * 	@param  string	$htmlname    	Name of html select
-     * 	@param	int		$modeproject		1 to restrict on projects owned by user
-     * 	@param	int		$modetask		1 to restrict on tasks associated to user
-     * 	@param	int		$mode			0=Return list of tasks and their projects, 1=Return projects and tasks if exists
-     *  @param  int		$useempty        0=Allow empty values
+     *	@param  int		$selectedtask   		Pre-selected task
+     *  @param  int		$projectid				Project id
+     * 	@param  string	$htmlname    			Name of html select
+     * 	@param	int		$modeproject			1 to restrict on projects owned by user
+     * 	@param	int		$modetask				1 to restrict on tasks associated to user
+     * 	@param	int		$mode					0=Return list of tasks and their projects, 1=Return projects and tasks if exists
+     *  @param  int		$useempty       		0=Allow empty values
+     *  @param	int		$disablechildoftaskid	1=Disable task that are child of the provided task id
      *  @return	void
      */
-    function selectProjectTasks($selectedtask='', $projectid=0, $htmlname='task_parent', $modeproject=0, $modetask=0, $mode=0, $useempty=0)
+    function selectProjectTasks($selectedtask='', $projectid=0, $htmlname='task_parent', $modeproject=0, $modetask=0, $mode=0, $useempty=0, $disablechildoftaskid=0)
     {
         global $user, $langs;
 
@@ -366,7 +450,7 @@ class FormOther
             if ($useempty) print '<option value="0">&nbsp;</option>';
             $j=0;
             $level=0;
-            $this->_pLineSelect($j, 0, $tasksarray, $level, $selectedtask, $projectid);
+            $this->_pLineSelect($j, 0, $tasksarray, $level, $selectedtask, $projectid, $disablechildoftaskid);
             print '</select>';
         }
         else
@@ -376,17 +460,18 @@ class FormOther
     }
 
     /**
-     * Write all lines of a project (if parent = 0)
+     * Write lines of a project (all lines of a project if parent = 0)
      *
      * @param 	int		&$inc					Cursor counter
-     * @param 	int		$parent					Id parent
-     * @param 	Object	$lines					Line object
+     * @param 	int		$parent					Id of parent task we want to see
+     * @param 	array	$lines					Array of task lines
      * @param 	int		$level					Level
      * @param 	int		$selectedtask			Id selected task
      * @param 	int		$selectedproject		Id selected project
+     * @param	int		$disablechildoftaskid	1=Disable task that are child of the provided task id
      * @return	void
      */
-    private function _pLineSelect(&$inc, $parent, $lines, $level=0, $selectedtask=0, $selectedproject=0)
+    private function _pLineSelect(&$inc, $parent, $lines, $level=0, $selectedtask=0, $selectedproject=0, $disablechildoftaskid=0)
     {
         global $langs, $user, $conf;
 
@@ -395,14 +480,16 @@ class FormOther
         $numlines=count($lines);
         for ($i = 0 ; $i < $numlines ; $i++)
         {
-            if ($lines[$i]->fk_parent == $parent)
+        	if ($lines[$i]->fk_parent == $parent)
             {
                 $var = !$var;
 
+				//var_dump($selectedproject."--".$selectedtask."--".$lines[$i]->fk_project."_".$lines[$i]->id);		// $lines[$i]->id may be empty if project has no lines
+
                 // Break on a new project
-                if ($parent == 0)
+                if ($parent == 0)	// We are on a task at first level
                 {
-                    if ($lines[$i]->fk_project != $lastprojectid)
+                    if ($lines[$i]->fk_project != $lastprojectid)	// Break found on project
                     {
                         if ($i > 0 && $conf->browser->firefox) print '<option value="0" disabled="disabled">----------</option>';
                         print '<option value="'.$lines[$i]->fk_project.'_0"';
@@ -425,11 +512,22 @@ class FormOther
                     }
                 }
 
+                $newdisablechildoftaskid=$disablechildoftaskid;
+
                 // Print task
-                if ($lines[$i]->id > 0)
+                if (isset($lines[$i]->id))		// We use isset because $lines[$i]->id may be null if project has no task and are on root project (tasks may be caught by a left join). We enter here only if '0' or >0
                 {
+                	// Check if we must disable entry
+                	$disabled=0;
+                	if ($disablechildoftaskid && (($lines[$i]->id == $disablechildoftaskid || $lines[$i]->fk_parent == $disablechildoftaskid)))
+                	{
+               			$disabled++;
+               			if ($lines[$i]->fk_parent == $disablechildoftaskid) $newdisablechildoftaskid=$lines[$i]->id;	// If task is child of a disabled parent, we will propagate id to disable next child too
+                	}
+
                     print '<option value="'.$lines[$i]->fk_project.'_'.$lines[$i]->id.'"';
-                    if ($lines[$i]->id == $selectedtask) print ' selected="selected"';
+                    if (($lines[$i]->id == $selectedtask) || ($lines[$i]->fk_project.'_'.$lines[$i]->id == $selectedtask)) print ' selected="selected"';
+                    if ($disabled) print ' disabled="disabled"';
                     print '>';
                     print $langs->trans("Project").' '.$lines[$i]->projectref;
                     if (empty($lines[$i]->public))
@@ -450,7 +548,7 @@ class FormOther
                 }
 
                 $level++;
-                if ($lines[$i]->id) $this->_pLineSelect($inc, $lines[$i]->id, $lines, $level, $selectedtask, $selectedproject);
+                if ($lines[$i]->id) $this->_pLineSelect($inc, $lines[$i]->id, $lines, $level, $selectedtask, $selectedproject, $newdisablechildoftaskid);
                 $level--;
             }
         }
@@ -465,6 +563,7 @@ class FormOther
      * 		@param	int			$showcolorbox	1=Show color code and color box, 0=Show only color code
      * 		@param 	array		$arrayofcolors	Array of colors. Example: array('29527A','5229A3','A32929','7A367A','B1365F','0D7813')
      * 		@return	void
+     * 		@deprecated
      */
     function select_color($set_color='', $prefix='f_color', $form_name='objForm', $showcolorbox=1, $arrayofcolors='')
     {
@@ -479,73 +578,79 @@ class FormOther
      *		@param	string		$form_name		Name of form
      * 		@param	int			$showcolorbox	1=Show color code and color box, 0=Show only color code
      * 		@param 	array		$arrayofcolors	Array of colors. Example: array('29527A','5229A3','A32929','7A367A','B1365F','0D7813')
+     * 		@param	string		$morecss		Add css style into input field
      * 		@return	void
      */
-    function selectColor($set_color='', $prefix='f_color', $form_name='objForm', $showcolorbox=1, $arrayofcolors='')
+    function selectColor($set_color='', $prefix='f_color', $form_name='objForm', $showcolorbox=1, $arrayofcolors='', $morecss='')
     {
-        global $langs;
+        global $langs,$conf;
 
         $out='';
 
         if (! is_array($arrayofcolors) || count($arrayofcolors) < 1)
         {
             $langs->load("other");
-            $out.= '<link rel="stylesheet" media="screen" type="text/css" href="'.DOL_URL_ROOT.'/includes/jquery/plugins/jpicker/css/jPicker-1.1.6.css" />';
-            $out.= '<script type="text/javascript" src="'.DOL_URL_ROOT.'/includes/jquery/plugins/jpicker/jpicker-1.1.6.js"></script>';
-            $out.= '<script type="text/javascript">
-             jQuery(document).ready(function(){
-                $(\'#colorpicker'.$prefix.'\').jPicker( {
-                window: {
-                  title: \''.dol_escape_js($langs->trans("SelectAColor")).'\', /* any title for the jPicker window itself - displays "Drag Markers To Pick A Color" if left null */
-                  effects:
-                    {
-                    type: \'show\', /* effect used to show/hide an expandable picker. Acceptable values "slide", "show", "fade" */
-                    speed:
-                    {
-                      show: \'fast\', /* duration of "show" effect. Acceptable values are "fast", "slow", or time in ms */
-                      hide: \'fast\' /* duration of "hide" effect. Acceptable values are "fast", "slow", or time in ms */
-                    }
-                    },
-                  position:
-                    {
-                    x: \'screenCenter\', /* acceptable values "left", "center", "right", "screenCenter", or relative px value */
-                    y: \'center\' /* acceptable values "top", "bottom", "center", or relative px value */
-                    },
-                },
-                images: {
-                    clientPath: \''.DOL_URL_ROOT.'/includes/jquery/plugins/jpicker/images/\',
-                    picker: { file: \'../../../../../theme/common/colorpicker.png\', width: 14, height: 14 }
-          		},
-                localization: // alter these to change the text presented by the picker (e.g. different language)
-                  {
-                    text:
-                    {
-                      title: \''.dol_escape_js($langs->trans("SelectAColor")).'\',
-                      newColor: \''.dol_escape_js($langs->trans("New")).'\',
-                      currentColor: \''.dol_escape_js($langs->trans("Current")).'\',
-                      ok: \''.dol_escape_js($langs->trans("Save")).'\',
-                      cancel: \''.dol_escape_js($langs->trans("Cancel")).'\'
-                    }
-                  }
-		        } ); });
-             </script>';
-            $out.= '<input id="colorpicker'.$prefix.'" name="'.$prefix.'" size="6" maxlength="7" class="flat" type="text" value="'.$set_color.'" />';
+            if (empty($conf->dol_use_jmobile))
+            {
+	            $out.= '<link rel="stylesheet" media="screen" type="text/css" href="'.DOL_URL_ROOT.'/includes/jquery/plugins/jpicker/css/jPicker-1.1.6.css" />';
+	            $out.= '<script type="text/javascript" src="'.DOL_URL_ROOT.'/includes/jquery/plugins/jpicker/jpicker-1.1.6.js"></script>';
+	            $out.= '<script type="text/javascript">
+	             jQuery(document).ready(function(){
+	                $(\'#colorpicker'.$prefix.'\').jPicker( {
+	                window: {
+	                  title: \''.dol_escape_js($langs->trans("SelectAColor")).'\', /* any title for the jPicker window itself - displays "Drag Markers To Pick A Color" if left null */
+	                  effects:
+	                    {
+	                    type: \'show\', /* effect used to show/hide an expandable picker. Acceptable values "slide", "show", "fade" */
+	                    speed:
+	                    {
+	                      show: \'fast\', /* duration of "show" effect. Acceptable values are "fast", "slow", or time in ms */
+	                      hide: \'fast\' /* duration of "hide" effect. Acceptable values are "fast", "slow", or time in ms */
+	                    }
+	                    },
+	                  position:
+	                    {
+	                    x: \'screenCenter\', /* acceptable values "left", "center", "right", "screenCenter", or relative px value */
+	                    y: \'center\' /* acceptable values "top", "bottom", "center", or relative px value */
+	                    },
+	                },
+	                images: {
+	                    clientPath: \''.DOL_URL_ROOT.'/includes/jquery/plugins/jpicker/images/\',
+	                    picker: { file: \'../../../../../theme/common/colorpicker.png\', width: 14, height: 14 }
+	          		},
+	                localization: // alter these to change the text presented by the picker (e.g. different language)
+	                  {
+	                    text:
+	                    {
+	                      title: \''.dol_escape_js($langs->trans("SelectAColor")).'\',
+	                      newColor: \''.dol_escape_js($langs->trans("New")).'\',
+	                      currentColor: \''.dol_escape_js($langs->trans("Current")).'\',
+	                      ok: \''.dol_escape_js($langs->trans("Save")).'\',
+	                      cancel: \''.dol_escape_js($langs->trans("Cancel")).'\'
+	                    }
+	                  }
+			        } ); });
+	             </script>';
+            }
+            $out.= '<input id="colorpicker'.$prefix.'" name="'.$prefix.'" size="6" maxlength="7" class="flat'.($morecss?' '.$morecss:'').'" type="text" value="'.$set_color.'" />';
         }
         else  // In most cases, this is not used. We used instead function with no specific list of colors
         {
-            $out.= '<link rel="stylesheet" href="'.DOL_URL_ROOT.'/includes/jquery/plugins/colorpicker/jquery.colorpicker.css" type="text/css" media="screen" />';
-            $out.= '<script src="'.DOL_URL_ROOT.'/includes/jquery/plugins/colorpicker/jquery.colorpicker.js" type="text/javascript"></script>';
-            $out.= '<script type="text/javascript">
-             jQuery(document).ready(function(){
-                 jQuery(\'#colorpicker'.$prefix.'\').colorpicker({
-                     size: 14,
-                     label: \'\',
-                     hide: true
-                 });
-             });
-             </script>';
-
-            $out.= '<select id="colorpicker'.$prefix.'" class="flat" name="'.$prefix.'">';
+            if (empty($conf->dol_use_jmobile))
+            {
+	        	$out.= '<link rel="stylesheet" href="'.DOL_URL_ROOT.'/includes/jquery/plugins/colorpicker/jquery.colorpicker.css" type="text/css" media="screen" />';
+	            $out.= '<script src="'.DOL_URL_ROOT.'/includes/jquery/plugins/colorpicker/jquery.colorpicker.js" type="text/javascript"></script>';
+	            $out.= '<script type="text/javascript">
+	             jQuery(document).ready(function(){
+	                 jQuery(\'#colorpicker'.$prefix.'\').colorpicker({
+	                     size: 14,
+	                     label: \'\',
+	                     hide: true
+	                 });
+	             });
+	             </script>';
+            }
+            $out.= '<select id="colorpicker'.$prefix.'" class="flat'.($morecss?' '.$morecss:'').'" name="'.$prefix.'">';
             //print '<option value="-1">&nbsp;</option>';
             foreach ($arrayofcolors as $val)
             {
@@ -809,32 +914,59 @@ class FormOther
         include_once DOL_DOCUMENT_ROOT.'/core/class/infobox.class.php';
 
         $confuserzone='MAIN_BOXES_'.$areacode;
-        
-        $boxactivated=InfoBox::listBoxes($db,'activated',$areacode,(empty($user->conf->$confuserzone)?null:$user));	// Search boxes of user (or everybody if user has no specific setup)
-         
+
+        $boxactivated=InfoBox::listBoxes($db,'activated',$areacode,(empty($user->conf->$confuserzone)?null:$user));	// Search boxes of common+user (or common only if user has no specific setup)
         $boxidactivatedforuser=array();
-        foreach($boxactivated as $box) 
+        foreach($boxactivated as $box)
         {
         	if (empty($user->conf->$confuserzone) || $box->fk_user == $user->id) $boxidactivatedforuser[$box->id]=$box->id;	// We keep only boxes to show for user
         }
-        
+
         $selectboxlist='';
         $arrayboxtoactivatelabel=array();
         if (! empty($user->conf->$confuserzone))
         {
+        	$langs->load("boxes");	// Load label of boxes
         	foreach($boxactivated as $box)
         	{
         		if (! empty($boxidactivatedforuser[$box->id])) continue;	// Already visible for user
-        		$arrayboxtoactivatelabel[$box->id]=$box->boxlabel;			// We keep only boxes not shown for user, to show into combo list
+        		$label=$langs->transnoentitiesnoconv($box->boxlabel);
+        		if (preg_match('/graph/',$box->class)) $label.=' ('.$langs->trans("Graph").')';
+        		$arrayboxtoactivatelabel[$box->id]=$label;			// We keep only boxes not shown for user, to show into combo list
         	}
 
         	$form=new Form($db);
             $selectboxlist=$form->selectarray('boxcombo', $arrayboxtoactivatelabel,'',1);
         }
 
+        // Javascript code for dynamic actions
         if (! empty($conf->use_javascript_ajax))
         {
 	        print '<script type="text/javascript" language="javascript">
+
+	        // To update list of activated boxes
+	        function updateBoxOrder(closing) {
+	        	var left_list = cleanSerialize(jQuery("#left").sortable("serialize"));
+	        	var right_list = cleanSerialize(jQuery("#right").sortable("serialize"));
+	        	var boxorder = \'A:\' + left_list + \'-B:\' + right_list;
+	        	if (boxorder==\'A:A-B:B\' && closing == 1)	// There is no more boxes on screen, and we are after a delete of a box so we must hide title
+	        	{
+	        		jQuery.ajax({
+	        			url: \''.DOL_URL_ROOT.'/core/ajax/box.php?boxorder=\'+boxorder+\'&zone='.$areacode.'&userid=\'+'.$user->id.',
+	        			async: false
+	        		});
+	        		// We force reload to be sure to get all boxes into list
+	        		window.location.search=\'mainmenu='.GETPOST("mainmenu").'&leftmenu='.GETPOST('leftmenu').'&action=delbox\';
+	        	}
+	        	else
+	        	{
+	        		jQuery.ajax({
+	        			url: \''.DOL_URL_ROOT.'/core/ajax/box.php?boxorder=\'+boxorder+\'&zone='.$areacode.'&userid=\'+'.$user->id.',
+	        			async: true
+	        		});
+	        	}
+	        }
+
 	        jQuery(document).ready(function() {
 	        	jQuery("#boxcombo").change(function() {
 	        	var boxid=jQuery("#boxcombo").val();
@@ -849,10 +981,33 @@ class FormOther
 	        			window.location.search=\'mainmenu='.GETPOST("mainmenu").'&leftmenu='.GETPOST('leftmenu').'&action=addbox&boxid=\'+boxid;
 	                }
 	        	});';
-	        if (! count($arrayboxtoactivatelabel)) print 'jQuery("#boxcombo").hide();';
-	        print  '
-	    	});
-	        </script>';
+	        	if (! count($arrayboxtoactivatelabel)) print 'jQuery("#boxcombo").hide();';
+	        	print  '
+
+	        	jQuery("#left, #right").sortable({
+		        	/* placeholder: \'ui-state-highlight\', */
+	    	    	handle: \'.boxhandle\',
+	    	    	revert: \'invalid\',
+	       			items: \'.box\',
+	        		containment: \'.fiche\',
+	        		connectWith: \'.connectedSortable\',
+	        		stop: function(event, ui) {
+	        			updateBoxOrder(0);
+	        		}
+	    		});
+
+	        	jQuery(".boxclose").click(function() {
+	        		var self = this;	// because JQuery can modify this
+	        		var boxid=self.id.substring(8);
+	        		var label=jQuery(\'#boxlabelentry\'+boxid).val();
+	        		jQuery(\'#boxto_\'+boxid).remove();
+	        		if (boxid > 0) jQuery(\'#boxcombo\').append(new Option(label, boxid));
+	        		updateBoxOrder(1);
+	        	});
+
+        	});'."\n";
+
+	        print '</script>'."\n";
         }
 
         $nbboxactivated=count($boxidactivatedforuser);
@@ -893,13 +1048,15 @@ class FormOther
                 }
             }
 
-            $emptybox->box_id='A';
-            $emptybox->info_box_head=array();
-            $emptybox->info_box_contents=array();
-            $emptybox->showBox(array(),array());
-
+            if (empty($conf->browser->phone))
+            {
+            	$emptybox->box_id='A';
+            	$emptybox->info_box_head=array();
+            	$emptybox->info_box_contents=array();
+            	$emptybox->showBox(array(),array());
+            }
             print "</div>\n";
-            print "<!-- End box container -->\n";
+            print "<!-- End box left container -->\n";
 
             print '</div><div class="fichehalfright"><div class="ficheaddleft">';
 
@@ -922,78 +1079,89 @@ class FormOther
                 }
             }
 
-            $emptybox->box_id='B';
-            $emptybox->info_box_head=array();
-            $emptybox->info_box_contents=array();
-            $emptybox->showBox(array(),array());
-
+            if (empty($conf->browser->phone))
+            {
+            	$emptybox->box_id='B';
+            	$emptybox->info_box_head=array();
+            	$emptybox->info_box_contents=array();
+            	$emptybox->showBox(array(),array());
+            }
             print "</div>\n";
-            print "<!-- End box container -->\n";
+            print "<!-- End box right container -->\n";
 
             print '</div></div>';
             print "\n";
 
             print "</td></tr>";
             print "</table>";
-
-            if ($conf->use_javascript_ajax)
-            {
-                print "\n";
-                print '<script type="text/javascript" language="javascript">';
-                // For moving
-                print 'jQuery(function() {
-                            jQuery("#left, #right").sortable({
-                                /* placeholder: \'ui-state-highlight\', */
-                                handle: \'.boxhandle\',
-                                revert: \'invalid\',
-                                items: \'.box\',
-                                containment: \'.fiche\',
-                                connectWith: \'.connectedSortable\',
-                                stop: function(event, ui) {
-                                    updateBoxOrder(0);
-                                }
-                            });
-                        });
-                '."\n";
-                // To update list of activated boxes
-                print 'function updateBoxOrder(closing) {
-	                		var left_list = cleanSerialize(jQuery("#left").sortable("serialize"));
-	                		var right_list = cleanSerialize(jQuery("#right").sortable("serialize"));
-	                		var boxorder = \'A:\' + left_list + \'-B:\' + right_list;
-	    					if (boxorder==\'A:A-B:B\' && closing == 1)	// There is no more boxes on screen, and we are after a delete of a box so we must hide title
-	    					{
-		    					jQuery.ajax({
-		    						url: \''.DOL_URL_ROOT.'/core/ajax/box.php?boxorder=\'+boxorder+\'&zone='.$areacode.'&userid=\'+'.$user->id.',
-		    						async: false
-		    					});
-	    						// We force reload to be sure to get all boxes into list
-			        			window.location.search=\'mainmenu='.GETPOST("mainmenu").'&leftmenu='.GETPOST('leftmenu').'&action=delbox\';
-	    					}
-	    					else
-	    					{
-	    						jQuery.ajax({
-		    						url: \''.DOL_URL_ROOT.'/core/ajax/box.php?boxorder=\'+boxorder+\'&zone='.$areacode.'&userid=\'+'.$user->id.',
-		    						async: true
-		    					});
-	    					}
-                		}'."\n";
-                // For closing
-                print 'jQuery(document).ready(function() {
-                          	jQuery(".boxclose").click(function() {
-                          		var self = this;	// because JQuery can modify this
-                              	var boxid=self.id.substring(8);
-                                jQuery(\'#boxto_\'+boxid).remove();
-                                updateBoxOrder(1);
-                           	});
-                       });'."\n";
-                print '</script>'."\n";
-            }
         }
 
         return count($boxactivated);
     }
 
 
+    /**
+     *  Return a HTML select list of bank accounts
+     *
+     *  @param  string	$htmlname          	Name of select zone
+     *  @param	string	$dictionnarytable	Dictionnary table
+     *  @param	string	$keyfield			Field for key
+     *  @param	string	$labelfield			Label field
+     *  @param	string	$selected			Selected value
+     *  @param  int		$useempty          	1=Add an empty value in list, 2=Add an empty value in list only if there is more than 2 entries.
+     * 	@return	void
+     */
+    function select_dictionnary($htmlname,$dictionnarytable,$keyfield='code',$labelfield='label',$selected='',$useempty=0)
+    {
+        global $langs, $conf;
+
+        $langs->load("admin");
+
+        $sql = "SELECT rowid, ".$keyfield.", ".$labelfield;
+        $sql.= " FROM ".MAIN_DB_PREFIX.$dictionnarytable;
+        $sql.= " ORDER BY ".$labelfield;
+
+        dol_syslog(get_class($this)."::select_dictionnary sql=".$sql);
+        $result = $this->db->query($sql);
+        if ($result)
+        {
+            $num = $this->db->num_rows($result);
+            $i = 0;
+            if ($num)
+            {
+                print '<select id="select'.$htmlname.'" class="flat selectdictionnary" name="'.$htmlname.'"'.($moreattrib?' '.$moreattrib:'').'>';
+                if ($useempty == 1 || ($useempty == 2 && $num > 1))
+                {
+                    print '<option value="-1">&nbsp;</option>';
+                }
+
+                while ($i < $num)
+                {
+                    $obj = $this->db->fetch_object($result);
+                    if ($selected == $obj->rowid || $selected == $obj->$keyfield)
+                    {
+                        print '<option value="'.$obj->$keyfield.'" selected="selected">';
+                    }
+                    else
+                    {
+                        print '<option value="'.$obj->$keyfield.'">';
+                    }
+                    print $obj->$labelfield;
+                    print '</option>';
+                    $i++;
+                }
+                print "</select>";
+            }
+            else
+			{
+                print $langs->trans("DictionnaryEmpty");
+            }
+        }
+        else {
+            dol_print_error($this->db);
+        }
+    }
+
 }
 
 ?>
