@@ -3,7 +3,7 @@
  * Copyright (C) 2004      Sebastien Di Cintio  <sdicintio@ressource-toi.org>
  * Copyright (C) 2004      Benoit Mortier       <benoit.mortier@opensides.be>
  * Copyright (C) 2004      Eric Seigne          <eric.seigne@ryxeo.com>
- * Copyright (C) 2005-2012 Laurent Destailleur  <eldy@users.sourceforge.net>
+ * Copyright (C) 2005-2013 Laurent Destailleur  <eldy@users.sourceforge.net>
  * Copyright (C) 2005-2012 Regis Houssin        <regis.houssin@capnetworks.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,13 +22,12 @@
 
 /**
  *  \file           htdocs/core/modules/DolibarrModules.class.php
- *  \brief          Fichier de description et activation des modules Dolibarr
+ *  \brief          File of parent class of module descriptor class files
  */
 
 
 /**
- *  \class      DolibarrModules
- *  \brief      Classe mere des classes de description et activation des modules Dolibarr
+ *  Parent class of module descriptor class files
  */
 abstract class DolibarrModules
 {
@@ -53,12 +52,13 @@ abstract class DolibarrModules
 
     var $dbversion = "-";
 
+
     /**
-     *      Fonction d'activation. Insere en base les constantes et boites du module
+     *  Method to enable a module. Insert into database all constants, boxes of module
      *
-     *      @param      array	$array_sql  Array of SQL requests to execute when enabling module
-     *      @param      string	$options    Options when enabling module ('', 'noboxes')
-     *      @return     int              	1 if OK, 0 if KO
+     *  @param      array	$array_sql  	Array of SQL requests to execute when enabling module
+     *  @param      string	$options    	String with options when disabling module ('newboxdefonly|noboxes')
+     *  @return     int              		1 if OK, 0 if KO
      */
     function _init($array_sql, $options='')
     {
@@ -79,8 +79,8 @@ abstract class DolibarrModules
         // Insert constant defined by modules, into llx_const
         if (! $err) $err+=$this->insert_const();
 
-        // Insere les boites dans llx_boxes_def
-        if (! $err && $options != 'noboxes') $err+=$this->insert_boxes();
+        // Insert boxes def into llx_boxes_def and boxes setup into llx_boxes
+        if (! $err && ! preg_match('/noboxes/',$options)) $err+=$this->insert_boxes($options);
 
         // Insert permission definitions of module into llx_rights_def. If user is admin, grant this permission to user.
         if (! $err) $err+=$this->insert_permissions(1);
@@ -114,7 +114,7 @@ abstract class DolibarrModules
                 $sql=str_replace('__ENTITY__', $conf->entity, $sql);
 
                 dol_syslog(get_class($this)."::_init ignoreerror=".$ignoreerror." sql=".$sql, LOG_DEBUG);
-                $result=$this->db->query($sql);
+                $result=$this->db->query($sql, $ignoreerror);
                 if (! $result)
                 {
                     if (! $ignoreerror)
@@ -148,7 +148,7 @@ abstract class DolibarrModules
      *  Fonction de desactivation. Supprime de la base les constantes et boites du module
      *
      *  @param      array	$array_sql      Array of SQL requests to execute when disable module
-     *  @param      string	$options		Options when disabling module ('', 'noboxes')
+     *  @param      string	$options		String with options when disabling module ('newboxdefonly|noboxes')
      *  @return     int      		       	1 if OK, 0 if KO
      */
     function _remove($array_sql, $options='')
@@ -171,7 +171,7 @@ abstract class DolibarrModules
         if (! $err) $err+=$this->delete_const();
 
         // Remove list of module's available boxes (entry in llx_boxes)
-        if (! $err && $options != 'noboxes') $err+=$this->delete_boxes();
+        if (! $err && ! preg_match('/(newboxdefonly|noboxes)/',$options)) $err+=$this->delete_boxes();	// We don't have to delete if option ask to keep boxes safe or ask to add new box def only
 
         // Remove module's permissions from list of available permissions (entries in llx_rights_def)
         if (! $err) $err+=$this->delete_permissions();
@@ -262,7 +262,7 @@ abstract class DolibarrModules
 
 
     /**
-     *  Retourne la version du module.
+     *  Return module version.
      *  Pour les modules a l'etat 'experimental', retourne la traduction de 'experimental'
      *  Pour les modules 'dolibarr', retourne la version de Dolibarr
      *  Pour les autres modules, retourne la version du module
@@ -282,6 +282,20 @@ abstract class DolibarrModules
     }
 
 
+    /**
+     *  Return if a module is a core or external module
+     *
+     *  @return     string      'core', 'external' or 'unknown'
+     */
+    function isCoreOrExternalModule()
+    {
+        if ($this->version == 'dolibarr') return 'core';
+        if (! empty($this->version) && ! in_array($this->version,array('experimental','development'))) return 'external';
+        if (! empty($this->editor_name) || ! empty($this->editor_web)) return 'external';
+        return 'unknown';
+    }
+
+
     /**
      *  Return list of lang files related to module
      *
@@ -433,7 +447,7 @@ abstract class DolibarrModules
                 if (is_resource($handle))
                 {
                 	$dirfound++;
-                	
+
 	                // Run llx_mytable.sql files
                 	while (($file = readdir($handle))!==false)
                     {
@@ -443,7 +457,7 @@ abstract class DolibarrModules
                             if ($result <= 0) $error++;
                         }
                     }
-                    
+
                     rewinddir($handle);
 
 	                // Run llx_mytable.key.sql files (Must be done after llx_mytable.sql)
@@ -457,7 +471,7 @@ abstract class DolibarrModules
                     }
 
                     rewinddir($handle);
-                    
+
                     // Run data_xxx.sql files (Must be done after llx_mytable.key.sql)
                 	while (($file = readdir($handle))!==false)
                     {
@@ -467,9 +481,9 @@ abstract class DolibarrModules
                             if ($result <= 0) $error++;
                         }
                     }
-                    
+
                     rewinddir($handle);
-                    
+
                     // Run update_xxx.sql files
                 	while (($file = readdir($handle))!==false)
                     {
@@ -479,7 +493,7 @@ abstract class DolibarrModules
                             if ($result <= 0) $error++;
                         }
                     }
-                    
+
                     closedir($handle);
                 }
 
@@ -498,11 +512,14 @@ abstract class DolibarrModules
     /**
      *  Insert boxes into llx_boxes_def
      *
-     *  @return     int     Nb of errors (0 if OK)
+     *	@param		string	$option		String with options when disabling module ('newboxdefonly'=insert only boxes definition)
+     *  @return     int     			Nb of errors (0 if OK)
      */
-    function insert_boxes()
+    function insert_boxes($option='')
     {
-        global $conf;
+		require_once DOL_DOCUMENT_ROOT.'/core/class/infobox.class.php';
+
+    	global $conf;
 
         $err=0;
 
@@ -510,15 +527,20 @@ abstract class DolibarrModules
         {
             foreach ($this->boxes as $key => $value)
             {
-                //$titre = $this->boxes[$key][0];
-                $file  = isset($this->boxes[$key][1])?$this->boxes[$key][1]:'';
-                $note  = isset($this->boxes[$key][2])?$this->boxes[$key][2]:'';
+                $file  = isset($this->boxes[$key]['file'])?$this->boxes[$key]['file']:'';
+                $note  = isset($this->boxes[$key]['note'])?$this->boxes[$key]['note']:'';
+                $enabledbydefaulton = isset($this->boxes[$key]['enabledbydefaulton'])?$this->boxes[$key]['enabledbydefaulton']:'Home';
 
+            	if (empty($file)) $file  = isset($this->boxes[$key][1])?$this->boxes[$key][1]:'';	// For backward compatibility
+                if (empty($note)) $note  = isset($this->boxes[$key][2])?$this->boxes[$key][2]:'';	// For backward compatibility
+
+                // Search if boxes def already present
                 $sql = "SELECT count(*) as nb FROM ".MAIN_DB_PREFIX."boxes_def";
                 $sql.= " WHERE file = '".$this->db->escape($file)."'";
                 $sql.= " AND entity = ".$conf->entity;
                 if ($note) $sql.=" AND note ='".$this->db->escape($note)."'";
 
+                dol_syslog(get_class($this)."::insert_boxes sql=".$sql);
                 $result=$this->db->query($sql);
                 if ($result)
                 {
@@ -529,9 +551,9 @@ abstract class DolibarrModules
 
                         if (! $err)
                         {
-                            $sql = "INSERT INTO ".MAIN_DB_PREFIX."boxes_def (file,entity,note)";
-                            $sql.= " VALUES ('".$this->db->escape($file)."',";
-                            $sql.= $conf->entity.",";
+                            $sql = "INSERT INTO ".MAIN_DB_PREFIX."boxes_def (file, entity, note)";
+                            $sql.= " VALUES ('".$this->db->escape($file)."', ";
+                            $sql.= $conf->entity.", ";
                             $sql.= $note?"'".$this->db->escape($note)."'":"null";
                             $sql.= ")";
 
@@ -540,16 +562,23 @@ abstract class DolibarrModules
                             if (! $resql) $err++;
 
                         }
-                        if (! $err)
+                        if (! $err && ! preg_match('/newboxdefonly/',$option))
                         {
                             $lastid=$this->db->last_insert_id(MAIN_DB_PREFIX."boxes_def","rowid");
 
-                            $sql = "INSERT INTO ".MAIN_DB_PREFIX."boxes (box_id,position,box_order,fk_user,entity)";
-                            $sql.= " VALUES (".$lastid.", 0, '0', 0, ".$conf->entity.")";
+                            $pos_name = getStaticMember('InfoBox','listOfPages');
+                            foreach ($pos_name as $key2 => $val2)
+                            {
+                            	//print 'key2='.$key2.'-val2='.$val2."<br>\n";
+                            	if ($enabledbydefaulton && $val2 != $enabledbydefaulton) continue;		// Not enabled by default onto this page.
 
-                            dol_syslog(get_class($this)."::insert_boxes sql=".$sql);
-                            $resql=$this->db->query($sql);
-                            if (! $resql) $err++;
+	                            $sql = "INSERT INTO ".MAIN_DB_PREFIX."boxes (box_id,position,box_order,fk_user,entity)";
+    	                        $sql.= " VALUES (".$lastid.", ".$key2.", '0', 0, ".$conf->entity.")";
+
+    	                        dol_syslog(get_class($this)."::insert_boxes onto page ".$key2."=".$val2." sql=".$sql);
+                            	$resql=$this->db->query($sql);
+                            	if (! $resql) $err++;
+                            }
                         }
 
                         if (! $err)
@@ -557,15 +586,16 @@ abstract class DolibarrModules
                             $this->db->commit();
                         }
                         else
-                        {
+						{
                             $this->error=$this->db->lasterror();
                             dol_syslog(get_class($this)."::insert_boxes ".$this->error, LOG_ERR);
                             $this->db->rollback();
                         }
                     }
+                    // else box already registered into database
                 }
                 else
-                {
+              {
                     $this->error=$this->db->lasterror();
                     dol_syslog(get_class($this)."::insert_boxes ".$this->error, LOG_ERR);
                     $err++;
@@ -593,9 +623,11 @@ abstract class DolibarrModules
             foreach ($this->boxes as $key => $value)
             {
                 //$titre = $this->boxes[$key][0];
-                $file  = $this->boxes[$key][1];
+                $file  = $this->boxes[$key]['file'];
                 //$note  = $this->boxes[$key][2];
 
+                if (empty($file)) $file  = isset($this->boxes[$key][1])?$this->boxes[$key][1]:'';	// For backward compatibility
+
                 $sql = "DELETE FROM ".MAIN_DB_PREFIX."boxes";
                 $sql.= " USING ".MAIN_DB_PREFIX."boxes, ".MAIN_DB_PREFIX."boxes_def";
                 $sql.= " WHERE ".MAIN_DB_PREFIX."boxes.box_id = ".MAIN_DB_PREFIX."boxes_def.rowid";
@@ -879,11 +911,12 @@ abstract class DolibarrModules
                         if ($this->db->errno() != "DB_ERROR_RECORD_ALREADY_EXISTS")
                         {
                             $this->error=$this->db->lasterror();
-                            dol_syslog(get_class($this)."::insert_permissions error ".$this->error, LOG_ERR);
+                            dol_syslog(get_class($this)."::insert_permissions errno = ".$this->db->errno()." error ".$this->error, LOG_ERR);
                             $err++;
                             break;
                         }
                         else dol_syslog(get_class($this)."::insert_permissions record already exists", LOG_INFO);
+
                     }
                     $this->db->free($resqlinsert);
 
@@ -992,7 +1025,7 @@ abstract class DolibarrModules
                 //print 'xxx'.$this->menu[$key]['fk_menu'];exit;
                 $foundparent=0;
                 $fk_parent=$this->menu[$key]['fk_menu'];
-                if (preg_match('/r=/',$fk_parent))
+                if (preg_match('/^r=/',$fk_parent))	// old deprecated method
                 {
                     $fk_parent=str_replace('r=','',$fk_parent);
                     if (isset($this->menu[$fk_parent]['rowid']))
@@ -1001,14 +1034,14 @@ abstract class DolibarrModules
                         $foundparent=1;
                     }
                 }
-                elseif (preg_match('/fk_mainmenu=(.*),fk_leftmenu=(.*)/',$fk_parent,$reg))
+                elseif (preg_match('/^fk_mainmenu=([a-zA-Z0-9_]+),fk_leftmenu=([a-zA-Z0-9_]+)$/',$fk_parent,$reg))
                 {
                     $menu->fk_menu=-1;
                     $menu->fk_mainmenu=$reg[1];
                     $menu->fk_leftmenu=$reg[2];
                     $foundparent=1;
                 }
-                elseif (preg_match('/fk_mainmenu=(.*)/',$fk_parent,$reg))
+                elseif (preg_match('/^fk_mainmenu=([a-zA-Z0-9_]+)$/',$fk_parent,$reg))
                 {
                     $menu->fk_menu=-1;
                     $menu->fk_mainmenu=$reg[1];
@@ -1223,14 +1256,14 @@ abstract class DolibarrModules
     /**
      * Insert activation of generic parts from modules in llx_const
 	 * Input entry use $this->module_parts = array(
-	 *                        	'triggers' => 0,                                 // Set this to 1 if module has its own trigger directory (core/triggers)
-	 *							'login' => 0,                                    // Set this to 1 if module has its own login method directory (core/login)
-	 *							'substitutions' => 0,                            // Set this to 1 if module has its own substitution function file (core/substitutions)
-	 *							'menus' => 0,                                    // Set this to 1 if module has its own menus handler directory (core/menus)
-	 *							'theme' => 0,                                    // Set this to 1 if module has its own theme directory (core/theme)
-	 *                        	'tpl' => 0,                                      // Set this to 1 if module overwrite template dir (core/tpl)
-	 *							'barcode' => 0,                                  // Set this to 1 if module has its own barcode directory (core/modules/barcode)
-	 *							'models' => 0,                                   // Set this to 1 if module has its own models directory (core/modules/xxx)
+	 *                        	'triggers' => 0,                                 // Set this to 1 if module has its own trigger directory (/mymodule/core/triggers)
+	 *							'login' => 0,                                    // Set this to 1 if module has its own login method directory (/mymodule/core/login)
+	 *							'substitutions' => 0,                            // Set this to 1 if module has its own substitution function file (/mymodule/core/substitutions)
+	 *							'menus' => 0,                                    // Set this to 1 if module has its own menus handler directory (/mymodule/core/menus)
+	 *							'theme' => 0,                                    // Set this to 1 if module has its own theme directory (/mymodule/theme)
+	 *                        	'tpl' => 0,                                      // Set this to 1 if module overwrite template dir (/mymodule/core/tpl)
+	 *							'barcode' => 0,                                  // Set this to 1 if module has its own barcode directory (/mymodule/core/modules/barcode)
+	 *							'models' => 0,                                   // Set this to 1 if module has its own models directory (/mymodule/core/modules/xxx)
 	 *							'css' => '/mymodule/css/mymodule.css.php',       // Set this to relative path of css file if module has its own css file
 	 *							'js' => '/mymodule/js/mymodule.js',              // Set this to relative path of js file if module must load a js on all pages
 	 *							'hooks' => array('hookcontext1','hookcontext2')  // Set here all hooks context managed by module
@@ -1250,6 +1283,8 @@ abstract class DolibarrModules
     	{
     		foreach($this->module_parts as $key => $value)
     		{
+    			if (is_array($value) && count($value) == 0) continue;	// Discard empty arrays
+
     			$newvalue = $value;
 
     			// Serialize array parameters
