@@ -2,7 +2,7 @@
 /* Copyright (C) 2002-2003 Rodolphe Quiedeville <rodolphe@quiedeville.org>
  * Copyright (C) 2004-2010 Laurent Destailleur  <eldy@users.sourceforge.net>
  * Copyright (C) 2005-2012 Regis Houssin        <regis.houssin@capnetworks.com>
- * Copyright (C) 2011	   Juanjo Menent        <jmenent@2byte.es>
+ * Copyright (C) 2011-2013 Juanjo Menent        <jmenent@2byte.es>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,6 +52,7 @@ class Fichinter extends CommonObject
 	var $note_private;
 	var $note_public;
 	var $fk_project;
+	var $fk_contrat;
 	var $modelpdf;
 	var $extraparams=array();
 
@@ -61,12 +62,13 @@ class Fichinter extends CommonObject
 	 *	Constructor
 	 *
 	 *  @param	DoliDB	$db		Database handler
-	 */
+ 	 */
 	function __construct($db)
 	{
 		$this->db = $db;
 		$this->products = array();
 		$this->fk_project = 0;
+		$this->fk_contrat = 0;
 		$this->statut = 0;
 
 		// List of language codes for status
@@ -82,16 +84,20 @@ class Fichinter extends CommonObject
 	/**
 	 *	Create an intervention into data base
 	 *
+	 *  @param		User	$user 		Objet user that make creation
+     *	@param		int		$notrigger	Disable all triggers
 	 *	@return		int		<0 if KO, >0 if OK
 	 */
-	function create()
+	function create($user, $notrigger=0)
 	{
 		global $conf, $user, $langs;
 
 		dol_syslog(get_class($this)."::create ref=".$this->ref);
 
 		// Check parameters
-		if (! is_numeric($this->duree)) { $this->duree = 0; }
+		if (! is_numeric($this->duree)) {
+			$this->duree = 0;
+		}
 		if ($this->socid <= 0)
 		{
 			$this->error='ErrorBadParameterForFunc';
@@ -133,6 +139,7 @@ class Fichinter extends CommonObject
 		$sql.= ", description";
 		$sql.= ", model_pdf";
 		$sql.= ", fk_projet";
+		$sql.= ", fk_contrat";
 		$sql.= ", fk_statut";
 		$sql.= ", note_private";
 		$sql.= ", note_public";
@@ -142,10 +149,11 @@ class Fichinter extends CommonObject
 		$sql.= ", '".$this->db->idate($now)."'";
 		$sql.= ", '".$this->ref."'";
 		$sql.= ", ".$conf->entity;
-		$sql.= ", ".$this->author;
+		$sql.= ", ".$user->id;
 		$sql.= ", ".($this->description?"'".$this->db->escape($this->description)."'":"null");
 		$sql.= ", '".$this->modelpdf."'";
 		$sql.= ", ".($this->fk_project ? $this->fk_project : 0);
+		$sql.= ", ".($this->fk_contrat ? $this->fk_contrat : 0);
 		$sql.= ", ".$this->statut;
 		$sql.= ", ".($this->note_private?"'".$this->db->escape($this->note_private)."'":"null");
 		$sql.= ", ".($this->note_public?"'".$this->db->escape($this->note_public)."'":"null");
@@ -157,23 +165,33 @@ class Fichinter extends CommonObject
 		{
 			$this->id=$this->db->last_insert_id(MAIN_DB_PREFIX."fichinter");
 
-            // Add linked object
-            if (! $error && $this->origin && $this->origin_id)
-            {
-                $ret = $this->add_object_linked();
-                if (! $ret)	dol_print_error($this->db);
-            }
+			if ($this->id)
+			{
+				$this->ref='(PROV'.$this->id.')';
+				$sql = 'UPDATE '.MAIN_DB_PREFIX."fichinter SET ref='".$this->ref."' WHERE rowid=".$this->id;
 
+				dol_syslog(get_class($this)."::create sql=".$sql);
+				$resql=$this->db->query($sql);
+				if (! $resql) $error++;
+			}
+			// Add linked object
+			if (! $error && $this->origin && $this->origin_id)
+			{
+				$ret = $this->add_object_linked();
+				if (! $ret)	dol_print_error($this->db);
+			}
+
+
+            if (! $notrigger)
+            {
 			// Appel des triggers
 			include_once DOL_DOCUMENT_ROOT . '/core/class/interfaces.class.php';
 			$interface=new Interfaces($this->db);
-			$tmpuser=new User($this->db);
-			$tmpuser->fetch($this->author);
-			$result=$interface->run_triggers('FICHINTER_CREATE',$this,$tmpuser,$langs,$conf);
+				$result=$interface->run_triggers('FICHINTER_CREATE',$this,$user,$langs,$conf);
 			if ($result < 0) {
 				$error++; $this->errors=$interface->errors;
 			}
-			// Fin appel triggers
+            }
 
 			if (! $error)
 			{
@@ -201,12 +219,18 @@ class Fichinter extends CommonObject
 	/**
 	 *	Update an intervention
 	 *
+	 *	@param		User	$user 		Objet user that make creation
+     *	@param		int		$notrigger	Disable all triggers
 	 *	@return		int		<0 if KO, >0 if OK
 	 */
-	function update()
+	function update($user, $notrigger=0)
 	{
-		if (! is_numeric($this->duree)) { $this->duree = 0; }
-		if (! dol_strlen($this->fk_project)) { $this->fk_project = 0; }
+	 	if (! is_numeric($this->duree)) {
+	 		$this->duree = 0;
+	 	}
+	 	if (! dol_strlen($this->fk_project)) {
+	 		$this->fk_project = 0;
+	 	}
 
 		$this->db->begin();
 
@@ -221,6 +245,9 @@ class Fichinter extends CommonObject
 		dol_syslog(get_class($this)."::update sql=".$sql, LOG_DEBUG);
 		if ($this->db->query($sql))
 		{
+
+			if (! $notrigger)
+			{
 			// Appel des triggers
 			include_once DOL_DOCUMENT_ROOT . '/core/class/interfaces.class.php';
 			$interface=new Interfaces($this->db);
@@ -229,6 +256,7 @@ class Fichinter extends CommonObject
 				$error++; $this->errors=$interface->errors;
 			}
 			// Fin appel triggers
+			}
 
 			$this->db->commit();
 			return 1;
@@ -286,9 +314,14 @@ class Fichinter extends CommonObject
 
 				if ($this->statut == 0) $this->brouillon = 1;
 
+				require_once(DOL_DOCUMENT_ROOT.'/core/class/extrafields.class.php');
+				$extrafields=new ExtraFields($this->db);
+				$extralabels=$extrafields->fetch_name_optionals_label($this->table_element,true);
+				$this->fetch_optionals($this->id,$extralabels);
+
 				/*
 				 * Lines
-				 */
+				*/
 				$result=$this->fetch_lines();
 				if ($result < 0)
 				{
@@ -351,6 +384,7 @@ class Fichinter extends CommonObject
 	function setValid($user)
 	{
 		global $langs, $conf;
+		require_once DOL_DOCUMENT_ROOT.'/core/lib/files.lib.php';
 
 		$error=0;
 
@@ -360,9 +394,20 @@ class Fichinter extends CommonObject
 
 			$now=dol_now();
 
+			// Define new ref
+			if (! $error && (preg_match('/^[\(]?PROV/i', $this->ref)))
+			{
+				$num = $this->getNextNumRef($this->thirdparty);
+			}
+			else
+			{
+				$num = $this->ref;
+			}
+
 			$sql = "UPDATE ".MAIN_DB_PREFIX."fichinter";
 			$sql.= " SET fk_statut = 1";
-			$sql.= ", date_valid = ".$this->db->idate($now);
+			$sql.= ", ref = '".$num."'";
+			$sql.= ", date_valid = '".$this->db->idate($now)."'";
 			$sql.= ", fk_user_valid = ".$user->id;
 			$sql.= " WHERE rowid = ".$this->id;
 			$sql.= " AND entity = ".$conf->entity;
@@ -370,32 +415,71 @@ class Fichinter extends CommonObject
 
 			dol_syslog(get_class($this)."::setValid sql=".$sql);
 			$resql=$this->db->query($sql);
-			if ($resql)
+			if (! $resql)
+			{
+				dol_syslog(get_class($this)."::setValid Echec update - 10 - sql=".$sql, LOG_ERR);
+				dol_print_error($this->db);
+				$error++;
+			}
+
+			if (! $error)
+			{
+				$this->oldref = '';
+
+				// Rename directory if dir was a temporary ref
+				if (preg_match('/^[\(]?PROV/i', $this->ref))
+				{
+					// Rename of object directory ($this->ref = old ref, $num = new ref)
+					// to  not lose the linked files
+					$oldref = dol_sanitizeFileName($this->ref);
+					$snum = dol_sanitizeFileName($num);
+					$dirsource = $conf->ficheinter->dir_output.'/'.$oldref;
+					$dirdest = $conf->ficheinter->dir_output.'/'.$snum;
+					if (file_exists($dirsource))
+					{
+						dol_syslog(get_class($this)."::validate rename dir ".$dirsource." into ".$dirdest);
+
+						if (@rename($dirsource, $dirdest))
+						{
+							$this->oldref = $oldref;
+
+							dol_syslog("Rename ok");
+							// Suppression ancien fichier PDF dans nouveau rep
+							dol_delete_file($conf->ficheinter->dir_output.'/'.$snum.'/'.$oldref.'*.*');
+						}
+					}
+				}
+			}
+
+			// Set new ref and define current statut
+			if (! $error)
+			{
+				$this->ref = $num;
+				$this->statut=1;
+				$this->brouillon=0;
+				$this->date_validation=$now;
+			}
+
+			if (! $error)
 			{
 				// Appel des triggers
 				include_once DOL_DOCUMENT_ROOT . '/core/class/interfaces.class.php';
 				$interface=new Interfaces($this->db);
 				$result=$interface->run_triggers('FICHINTER_VALIDATE',$this,$user,$langs,$conf);
-				if ($result < 0) { $error++; $this->errors=$interface->errors; }
+	 			if ($result < 0) {
+	 				$error++; $this->errors=$interface->errors;
+	 			}
 				// Fin appel triggers
+			}
 
-				if (! $error)
-				{
-					$this->db->commit();
-					return 1;
-				}
-				else
-				{
-					$this->db->rollback();
-					$this->error=join(',',$this->errors);
-					dol_syslog(get_class($this)."::setValid ".$this->error,LOG_ERR);
-					return -1;
-				}
+			if (! $error)
+			{
+				$this->db->commit();
+				return 1;
 			}
 			else
 			{
 				$this->db->rollback();
-				$this->error=$this->db->lasterror();
 				dol_syslog(get_class($this)."::setValid ".$this->error,LOG_ERR);
 				return -1;
 			}
@@ -478,19 +562,20 @@ class Fichinter extends CommonObject
 		}
 		if ($mode == 5)
 		{
-			if ($statut==0) return $langs->trans($this->statuts_short[$statut]).' '.img_picto($langs->trans($this->statuts_short[$statut]),'statut0');
-			if ($statut==1) return $langs->trans($this->statuts_short[$statut]).' '.img_picto($langs->trans($this->statuts_short[$statut]),'statut4');
-			if ($statut==2) return $langs->trans($this->statuts_short[$statut]).' '.img_picto($langs->trans($this->statuts_short[$statut]),'statut6');
+			if ($statut==0) return '<span class="hideonsmartphone">'.$langs->trans($this->statuts_short[$statut]).' </span>'.img_picto($langs->trans($this->statuts_short[$statut]),'statut0');
+			if ($statut==1) return '<span class="hideonsmartphone">'.$langs->trans($this->statuts_short[$statut]).' </span>'.img_picto($langs->trans($this->statuts_short[$statut]),'statut4');
+			if ($statut==2) return '<span class="hideonsmartphone">'.$langs->trans($this->statuts_short[$statut]).' </span>'.img_picto($langs->trans($this->statuts_short[$statut]),'statut6');
 		}
 	}
 
 	/**
 	 *	Return clicable name (with picto eventually)
 	 *
-	 *	@param		int			$withpicto		0=_No picto, 1=Includes the picto in the linkn, 2=Picto only
-	 *	@return		string						String with URL
+	 *	@param		int		$withpicto		0=_No picto, 1=Includes the picto in the linkn, 2=Picto only
+	 *	@param		string	$option			Options
+	 *	@return		string					String with URL
 	 */
-	function getNomUrl($withpicto=0)
+	function getNomUrl($withpicto=0,$option='')
 	{
 		global $langs;
 
@@ -560,9 +645,9 @@ class Fichinter extends CommonObject
 	}
 
 	/**
-	 * 	Information sur l'objet fiche intervention
+	 * 	Load information on object
 	 *
-	 *	@param	int		$id      Id de la fiche d'intervention
+	 *	@param	int		$id      Id of object
 	 *	@return	void
 	 */
 	function info($id)
@@ -578,13 +663,12 @@ class Fichinter extends CommonObject
 		$sql.= " WHERE f.rowid = ".$id;
 		$sql.= " AND f.entity = ".$conf->entity;
 
-		$result = $this->db->query($sql);
-
-		if ($result)
+		$resql = $this->db->query($sql);
+		if ($resql)
 		{
-			if ($this->db->num_rows($result))
+			if ($this->db->num_rows($resql))
 			{
-				$obj = $this->db->fetch_object($result);
+				$obj = $this->db->fetch_object($resql);
 
 				$this->id                = $obj->rowid;
 
@@ -602,7 +686,7 @@ class Fichinter extends CommonObject
 					$this->user_validation     = $vuser;
 				}
 			}
-			$this->db->free($result);
+			$this->db->free($resql);
 		}
 		else
 		{
@@ -613,12 +697,13 @@ class Fichinter extends CommonObject
 	/**
 	 *	Delete intervetnion
 	 *
-	 *	@param      User	$user	Object user who delete
-	 *	@return		int				<0 if KO, >0 if OK
+	 *	@param      User	$user			Object user who delete
+	 *	@param		int		$notrigger		Disable trigger
+	 *	@return		int						<0 if KO, >0 if OK
 	 */
-	function delete($user)
+	function delete($user, $notrigger=0)
 	{
-		global $conf;
+		global $conf,$langs;
         require_once DOL_DOCUMENT_ROOT.'/core/lib/files.lib.php';
 
 		$error=0;
@@ -626,8 +711,8 @@ class Fichinter extends CommonObject
 		$this->db->begin();
 
 		// Delete linked object
-        $res = $this->deleteObjectLinked();
-        if ($res < 0) $error++;
+		$res = $this->deleteObjectLinked();
+		if ($res < 0) $error++;
 
 		// Delete linked contacts
 		$res = $this->delete_linked_contact();
@@ -683,14 +768,17 @@ class Fichinter extends CommonObject
 					}
 				}
 
-				// Appel des triggers
-				include_once DOL_DOCUMENT_ROOT . '/core/class/interfaces.class.php';
-				$interface=new Interfaces($this->db);
-				$result=$interface->run_triggers('FICHINTER_DELETE',$this,$user,$langs,$conf);
-				if ($result < 0) {
-					$error++; $this->errors=$interface->errors;
+				if (! $notrigger)
+				{
+					// Appel des triggers
+					include_once DOL_DOCUMENT_ROOT . '/core/class/interfaces.class.php';
+					$interface=new Interfaces($this->db);
+					$result=$interface->run_triggers('FICHINTER_DELETE',$this,$user,$langs,$conf);
+					if ($result < 0) {
+						$error++; $this->errors=$interface->errors;
+					}
+					// Fin appel triggers
 				}
-				// Fin appel triggers
 
 				$this->db->commit();
 				return 1;
@@ -776,16 +864,52 @@ class Fichinter extends CommonObject
 		}
 	}
 
+
+	/**
+	 *	Link intervention to a contract
+	 *
+	 *	@param      User	$user			Object user who modify
+	 *	@param      int		$contractid		Description
+	 *	@return     int						<0 if ko, >0 if ok
+	 */
+	function set_contrat($user, $contractid)
+	{
+		global $conf;
+
+		if ($user->rights->ficheinter->creer)
+		{
+			$sql = "UPDATE ".MAIN_DB_PREFIX."fichinter ";
+			$sql.= " SET fk_contrat = '".$contractid."'";
+			$sql.= " WHERE rowid = ".$this->id;
+			$sql.= " AND entity = ".$conf->entity;
+
+			dol_syslog("sql=".$sql);
+			if ($this->db->query($sql))
+			{
+				$this->fk_contrat = $contractid;
+				return 1;
+			}
+			else
+			{
+				$this->error=$this->db->error();
+				dol_syslog($this->error, LOG_ERR);
+				return -1;
+			}
+		}
+		return -2;
+	}
+
 	/**
 	 *	Adding a line of intervention into data base
 	 *
+	 *  @param      user	$user					User that do the action
 	 *	@param    	int		$fichinterid			Id of intervention
 	 *	@param    	string	$desc					Line description
 	 *	@param      date	$date_intervention  	Intervention date
 	 *	@param      int		$duration            	Intervention duration
 	 *	@return    	int             				>0 if ok, <0 if ko
 	 */
-	function addline($fichinterid, $desc, $date_intervention, $duration)
+	function addline($user,$fichinterid, $desc, $date_intervention, $duration)
 	{
 		dol_syslog("Fichinter::Addline $fichinterid, $desc, $date_intervention, $duration");
 
@@ -801,7 +925,7 @@ class Fichinter extends CommonObject
 			$line->datei        = $date_intervention;
 			$line->duration     = $duration;
 
-			$result=$line->insert();
+			$result=$line->insert($user);
 			if ($result > 0)
 			{
 				$this->db->commit();
@@ -819,11 +943,11 @@ class Fichinter extends CommonObject
 
 
 	/**
-     *  Initialise an instance with random values.
-     *  Used to build previews or test instances.
-     *	id must be 0 if object instance is a specimen.
-     *
-     *  @return	void
+	 *  Initialise an instance with random values.
+	 *  Used to build previews or test instances.
+	 *	id must be 0 if object instance is a specimen.
+	 *
+	 *  @return	void
 	 */
 	function initAsSpecimen()
 	{
@@ -848,7 +972,7 @@ class Fichinter extends CommonObject
 			$line->desc=$langs->trans("Description")." ".$xnbp;
 			$line->datei=($now-3600*(1+$xnbp));
 			$line->duration=600;
-            $line->fk_fichinter=0;
+			$line->fk_fichinter=0;
 			$this->lines[$xnbp]=$line;
 			$xnbp++;
 
@@ -968,10 +1092,14 @@ class FichinterLigne
 	/**
 	 *	Insert the line into database
 	 *
+	 *	@param		User	$user 		Objet user that make creation
+     *	@param		int		$notrigger	Disable all triggers
 	 *	@return		int		<0 if ko, >0 if ok
 	 */
-	function insert()
+	function insert($user, $notrigger=0)
 	{
+		global $langs,$conf;
+
 		dol_syslog("FichinterLigne::insert rang=".$this->rang);
 
 		$this->db->begin();
@@ -1001,7 +1129,7 @@ class FichinterLigne
 		$sql.= ' (fk_fichinter, description, date, duree, rang)';
 		$sql.= " VALUES (".$this->fk_fichinter.",";
 		$sql.= " '".$this->db->escape($this->desc)."',";
-		$sql.= " ".$this->db->idate($this->datei).",";
+		$sql.= " '".$this->db->idate($this->datei)."',";
 		$sql.= " ".$this->duration.",";
 		$sql.= ' '.$rangToUse;
 		$sql.= ')';
@@ -1014,6 +1142,21 @@ class FichinterLigne
 			if ($result > 0)
 			{
 				$this->rang=$rangToUse;
+
+				if (! $notrigger)
+				{
+					// Appel des triggers
+					include_once DOL_DOCUMENT_ROOT . '/core/class/interfaces.class.php';
+					$interface=new Interfaces($this->db);
+					$resulttrigger=$interface->run_triggers('LINEFICHINTER_CREATE',$this,$user,$langs,$conf);
+					if ($resulttrigger < 0) {
+						$error++; $this->errors=$interface->errors;
+					}
+					// Fin appel triggers
+				}
+			}
+
+			if (!$error) {
 				$this->db->commit();
 				return $result;
 			}
@@ -1036,10 +1179,14 @@ class FichinterLigne
 	/**
 	 *	Update intervention into database
 	 *
+	 *	@param		User	$user 		Objet user that make creation
+     *	@param		int		$notrigger	Disable all triggers
 	 *	@return		int		<0 if ko, >0 if ok
 	 */
-	function update()
+	function update($user,$notrigger=0)
 	{
+		global $langs,$conf;
+
 		$this->db->begin();
 
 		// Mise a jour ligne en base
@@ -1056,6 +1203,22 @@ class FichinterLigne
 		{
 			$result=$this->update_total();
 			if ($result > 0)
+			{
+
+				if (! $notrigger)
+				{
+					// Appel des triggers
+					include_once DOL_DOCUMENT_ROOT . '/core/class/interfaces.class.php';
+					$interface=new Interfaces($this->db);
+					$resulttrigger=$interface->run_triggers('LINEFICHINTER_UPDATE',$this,$user,$langs,$conf);
+					if ($resulttrigger < 0) {
+						$error++; $this->errors=$interface->errors;
+					}
+					// Fin appel triggers
+				}
+			}
+
+			if (!$error)
 			{
 				$this->db->commit();
 				return $result;
@@ -1132,10 +1295,14 @@ class FichinterLigne
 	/**
 	 *	Delete a intervention line
 	 *
+	 *	@param		User	$user 		Objet user that make creation
+     *	@param		int		$notrigger	Disable all triggers
 	 *	@return     int		>0 if ok, <0 if ko
 	 */
-	function deleteline()
+	function deleteline($user,$notrigger=0)
 	{
+		global $langs,$conf;
+
 		if ($this->statut == 0)
 		{
 			dol_syslog(get_class($this)."::deleteline lineid=".$this->rowid);
@@ -1151,6 +1318,19 @@ class FichinterLigne
 				if ($result > 0)
 				{
 					$this->db->commit();
+
+					if (! $notrigger)
+					{
+						// Appel des triggers
+						include_once DOL_DOCUMENT_ROOT . '/core/class/interfaces.class.php';
+						$interface=new Interfaces($this->db);
+						$resulttrigger=$interface->run_triggers('LINEFICHINTER_DELETE',$this,$user,$langs,$conf);
+						if ($resulttrigger < 0) {
+							$error++; $this->errors=$interface->errors;
+						}
+						// Fin appel triggers
+					}
+
 					return $result;
 				}
 				else
