@@ -1,7 +1,8 @@
 <?php
-/* Copyright (C) 2011	Dimitri Mouillard	<dmouillard@teclib.com>
- * Copyright (C) 2012	Laurent Destailleur	<eldy@users.sourceforge.net>
- * Copyright (C) 2012	Regis Houssin		<regis.houssin@capnetworks.com>
+/* Copyright (C) 2011	   Dimitri Mouillard	<dmouillard@teclib.com>
+ * Copyright (C) 2012-2014 Laurent Destailleur	<eldy@users.sourceforge.net>
+ * Copyright (C) 2012	   Regis Houssin		<regis.houssin@capnetworks.com>
+ * Copyright (C) 2013      Florian Henry        <florian.henry@open-concept.pro>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +47,7 @@ class Holiday extends CommonObject
     var $date_debut='';
     var $date_fin='';
     var $halfday='';
-    var $statut='';
+    var $statut='';			// 1=draft, 2=validated, 3=approved
     var $fk_validator;
     var $date_valid='';
     var $fk_user_valid;
@@ -86,7 +87,7 @@ class Holiday extends CommonObject
 	    $this->updateSoldeCP();
 
 	    // Vérifie le nombre d'utilisateur et mets à jour si besoin
-	    $this->verifNbUsers($this->countActiveUsers(),$this->getConfCP('nbUser'));
+	    $this->verifNbUsers($this->countActiveUsersWithoutCP(),$this->getConfCP('nbUser'));
 	    return 1;
     }
 
@@ -123,7 +124,7 @@ class Holiday extends CommonObject
         // User
         $sql.= "'".$this->fk_user."',";
         $sql.= " '".$this->db->idate($now)."',";
-        $sql.= " '".addslashes($this->description)."',";
+        $sql.= " '".$this->db->escape($this->description)."',";
         $sql.= " '".$this->db->idate($this->date_debut)."',";
         $sql.= " '".$this->db->idate($this->date_fin)."',";
         $sql.= " ".$this->halfday.",";
@@ -177,7 +178,6 @@ class Holiday extends CommonObject
 
         $sql = "SELECT";
         $sql.= " cp.rowid,";
-
         $sql.= " cp.fk_user,";
         $sql.= " cp.date_create,";
         $sql.= " cp.description,";
@@ -192,8 +192,9 @@ class Holiday extends CommonObject
         $sql.= " cp.fk_user_refuse,";
         $sql.= " cp.date_cancel,";
         $sql.= " cp.fk_user_cancel,";
-        $sql.= " cp.detail_refuse";
-
+        $sql.= " cp.detail_refuse,";
+        $sql.= " cp.note_private,";
+        $sql.= " cp.note_public";
         $sql.= " FROM ".MAIN_DB_PREFIX."holiday as cp";
         $sql.= " WHERE cp.rowid = ".$id;
 
@@ -223,6 +224,8 @@ class Holiday extends CommonObject
                 $this->date_cancel = $this->db->jdate($obj->date_cancel);
                 $this->fk_user_cancel = $obj->fk_user_cancel;
                 $this->detail_refuse = $obj->detail_refuse;
+                $this->note_private = $obj->note_private;
+                $this->note_public = $obj->note_public;
             }
             $this->db->free($resql);
 
@@ -265,10 +268,17 @@ class Holiday extends CommonObject
         $sql.= " cp.fk_user_refuse,";
         $sql.= " cp.date_cancel,";
         $sql.= " cp.fk_user_cancel,";
-        $sql.= " cp.detail_refuse";
+        $sql.= " cp.detail_refuse,";
 
-        $sql.= " FROM ".MAIN_DB_PREFIX."holiday as cp";
-        $sql.= " WHERE cp.fk_user = '".$user_id."'";
+		$sql.= " uu.lastname as user_lastname,";
+        $sql.= " uu.firstname as user_firstname,";
+
+        $sql.= " ua.lastname as validator_lastname,";
+        $sql.= " ua.firstname as validator_firstname";
+
+        $sql.= " FROM ".MAIN_DB_PREFIX."holiday as cp, ".MAIN_DB_PREFIX."user as uu, ".MAIN_DB_PREFIX."user as ua";
+		$sql.= " WHERE cp.fk_user = uu.rowid AND cp.fk_validator = ua.rowid "; // Hack pour la recherche sur le tableau
+        $sql.= " AND cp.fk_user = '".$user_id."'";
 
         // Filtre de séléction
         if(!empty($filter)) {
@@ -318,6 +328,12 @@ class Holiday extends CommonObject
                 $tab_result[$i]['fk_user_cancel'] = $obj->fk_user_cancel;
                 $tab_result[$i]['detail_refuse'] = $obj->detail_refuse;
 
+                $tab_result[$i]['user_firstname'] = $obj->user_firstname;
+                $tab_result[$i]['user_lastname'] = $obj->user_lastname;
+
+                $tab_result[$i]['validator_firstname'] = $obj->validator_firstname;
+                $tab_result[$i]['validator_lastname'] = $obj->validator_lastname;
+
                 $i++;
             }
 
@@ -364,10 +380,10 @@ class Holiday extends CommonObject
         $sql.= " cp.fk_user_cancel,";
         $sql.= " cp.detail_refuse,";
 
-        $sql.= " uu.name as user_lastname,";
+        $sql.= " uu.lastname as user_lastname,";
         $sql.= " uu.firstname as user_firstname,";
 
-        $sql.= " ua.name as validator_lastname,";
+        $sql.= " ua.lastname as validator_lastname,";
         $sql.= " ua.firstname as validator_firstname";
 
         $sql.= " FROM ".MAIN_DB_PREFIX."holiday as cp, ".MAIN_DB_PREFIX."user as uu, ".MAIN_DB_PREFIX."user as ua";
@@ -457,7 +473,7 @@ class Holiday extends CommonObject
         // Update request
         $sql = "UPDATE ".MAIN_DB_PREFIX."holiday SET";
 
-        $sql.= " description= '".addslashes($this->description)."',";
+        $sql.= " description= '".$this->db->escape($this->description)."',";
 
         if(!empty($this->date_debut)) {
             $sql.= " date_debut = '".$this->db->idate($this->date_debut)."',";
@@ -511,7 +527,7 @@ class Holiday extends CommonObject
             $sql.= " fk_user_cancel = NULL,";
         }
         if(!empty($this->detail_refuse)) {
-            $sql.= " detail_refuse = '".addslashes($this->detail_refuse)."'";
+            $sql.= " detail_refuse = '".$this->db->escape($this->detail_refuse)."'";
         } else {
             $sql.= " detail_refuse = NULL";
         }
@@ -855,7 +871,7 @@ class Holiday extends CommonObject
      *
      *	@param		int		$userID		Id of user
      *	@param		int		$nbHoliday	Nb of days
-     *  @return     void
+     *  @return     int					0=Nothing done, 1=OK, -1=KO
      */
     function updateSoldeCP($userID='',$nbHoliday='')
     {
@@ -891,13 +907,13 @@ class Holiday extends CommonObject
 
                 $i = 0;
 
-                while($i < $nbUser)
+                while ($i < $nbUser)
                 {
                     $now_holiday = $this->getCPforUser($users[$i]['rowid']);
                     $new_solde = $now_holiday + $this->getConfCP('nbHolidayEveryMonth');
 
                     // On ajoute la modification dans le LOG
-                    $this->addLogCP($user->id,$users[$i]['rowid'], $langs->trans('Event').': '.$langs->trans('HolidaysMonthlyUpdate'),$new_solde);
+                    $this->addLogCP($user->id,$users[$i]['rowid'], $langs->trans('HolidaysMonthlyUpdate'),$new_solde);
 
                     $i++;
                 }
@@ -906,11 +922,18 @@ class Holiday extends CommonObject
                 $sql2.= " nb_holiday = nb_holiday + ".$nb_holiday;
 
                 dol_syslog(get_class($this).'::updateSoldeCP sql='.$sql2);
-                $this->db->query($sql2);
+                $result= $this->db->query($sql2);
+
+	            if ($result) return 1;
+    	        else return -1;
             }
-        } else {
+
+            return 0;
+        }
+        else
+        {
             // Mise à jour pour un utilisateur
-            $nbHoliday = number_format($nbHoliday,2,'.','');
+            $nbHoliday = price2num($nbHoliday,2);
 
             // Mise à jour pour un utilisateur
             $sql = "UPDATE ".MAIN_DB_PREFIX."holiday_users SET";
@@ -918,7 +941,10 @@ class Holiday extends CommonObject
             $sql.= " WHERE fk_user = '".$userID."'";
 
 			dol_syslog(get_class($this).'::updateSoldeCP sql='.$sql);
-            $this->db->query($sql);
+            $result = $this->db->query($sql);
+
+            if ($result) return 1;
+            else return -1;
         }
 
     }
@@ -1035,7 +1061,7 @@ class Holiday extends CommonObject
 
                 $sql = "SELECT u.rowid";
                 $sql.= " FROM ".MAIN_DB_PREFIX."user as u";
-                $sql.= " WHERE statut > '0'";
+                $sql.= " WHERE statut > 0";
 
                 dol_syslog(get_class($this)."::fetchUsers sql=".$sql, LOG_DEBUG);
                 $resql=$this->db->query($sql);
@@ -1115,9 +1141,9 @@ class Holiday extends CommonObject
             // Si c'est pour les utilisateurs de Dolibarr
             if($type) {
 
-                $sql = "SELECT u.rowid, u.name, u.firstname";
+                $sql = "SELECT u.rowid, u.lastname, u.firstname";
                 $sql.= " FROM ".MAIN_DB_PREFIX."user as u";
-                $sql.= " WHERE statut > '0'";
+                $sql.= " WHERE statut > 0";
 
                 dol_syslog(get_class($this)."::fetchUsers sql=".$sql, LOG_DEBUG);
                 $resql=$this->db->query($sql);
@@ -1135,7 +1161,7 @@ class Holiday extends CommonObject
                         $obj = $this->db->fetch_object($resql);
 
                         $tab_result[$i]['rowid'] = $obj->rowid;
-                        $tab_result[$i]['name'] = $obj->name;
+                        $tab_result[$i]['name'] = $obj->lastname;
                         $tab_result[$i]['firstname'] = $obj->firstname;
 
                         $i++;
@@ -1154,7 +1180,7 @@ class Holiday extends CommonObject
                 // Si c'est pour les utilisateurs du module Congés Payés
             } else {
 
-                $sql = "SELECT cpu.fk_user, u.name, u.firstname";
+                $sql = "SELECT cpu.fk_user, u.lastname, u.firstname";
                 $sql.= " FROM ".MAIN_DB_PREFIX."holiday_users as cpu,";
                 $sql.= " ".MAIN_DB_PREFIX."user as u";
                 $sql.= " WHERE cpu.fk_user = u.rowid";
@@ -1175,7 +1201,7 @@ class Holiday extends CommonObject
                         $obj = $this->db->fetch_object($resql);
 
                         $tab_result[$i]['rowid'] = $obj->fk_user;
-                        $tab_result[$i]['name'] = $obj->name;
+                        $tab_result[$i]['name'] = $obj->lastname;
                         $tab_result[$i]['firstname'] = $obj->firstname;
 
                         $i++;
@@ -1203,7 +1229,23 @@ class Holiday extends CommonObject
 
         $sql = "SELECT count(u.rowid) as compteur";
         $sql.= " FROM ".MAIN_DB_PREFIX."user as u";
-        $sql.= " WHERE statut > '0'";
+		$sql.= " WHERE u.statut > 0";
+
+        $result = $this->db->query($sql);
+        $objet = $this->db->fetch_object($result);
+        return $objet->compteur;
+
+    }
+    /**
+     *	Compte le nombre d'utilisateur actifs dans Dolibarr sans CP
+     *
+     *  @return     int      retourne le nombre d'utilisateur
+     */
+    function countActiveUsersWithoutCP() {
+
+        $sql = "SELECT count(u.rowid) as compteur";
+        $sql.= " FROM ".MAIN_DB_PREFIX."user as u LEFT OUTER JOIN ".MAIN_DB_PREFIX."holiday_users hu ON (hu.fk_user=u.rowid)";
+		$sql.= " WHERE u.statut > 0 AND hu.fk_user IS NULL ";
 
         $result = $this->db->query($sql);
         $objet = $this->db->fetch_object($result);
@@ -1214,24 +1256,24 @@ class Holiday extends CommonObject
     /**
      *  Compare le nombre d'utilisateur actif de Dolibarr à celui des utilisateurs des congés payés
      *
-     *  @param    int	$userDolibarr	nombre d'utilisateur actifs dans Dolibarr
-     *  @param    int	$userCP    		nombre d'utilisateur actifs dans le module congés payés
+     *  @param    int	$userDolibarrWithoutCP	Number of active users in Dolibarr without holidays
+     *  @param    int	$userCP    				Number of active users into table of holidays
      *  @return   void
      */
-    function verifNbUsers($userDolibarr,$userCP) {
+    function verifNbUsers($userDolibarrWithoutCP,$userCP) {
 
     	if (empty($userCP)) $userCP=0;
     	dol_syslog(get_class($this).'::verifNbUsers userDolibarr='.$userDolibarr.' userCP='.$userCP);
 
-        // Si il y a plus d'utilisateur Dolibarr que dans le module CP
-        if ($userDolibarr > $userCP)
+        // On vérifie les users Dolibarr sans CP
+        if ($userDolibarrWithoutCP > 0)
         {
             $this->updateConfCP('nbUser',$userDolibarr);
 
             $listUsersCP = $this->fetchUsers(true,false);
 
             // On séléctionne les utilisateurs qui ne sont pas déjà dans le module
-            $sql = "SELECT u.rowid, u.name, u.firstname";
+            $sql = "SELECT u.rowid, u.lastname, u.firstname";
             $sql.= " FROM ".MAIN_DB_PREFIX."user as u";
             $sql.= " WHERE u.rowid NOT IN(".$listUsersCP.")";
 
@@ -1285,7 +1327,7 @@ class Holiday extends CommonObject
 
                     $obj = $this->db->fetch_object($resql);
 
-                    // On ajoute l'utilisateur
+                    // On supprime l'utilisateur
                     $this->deleteCPuser($obj->fk_user);
 
                     $i++;
@@ -1378,7 +1420,7 @@ class Holiday extends CommonObject
 
         $sql.= ") VALUES (";
 
-        $sql.= " '".addslashes($this->optName)."',";
+        $sql.= " '".$this->db->escape($this->optName)."',";
         $sql.= " '".$this->optValue."'";
         $sql.= ")";
 
@@ -1425,7 +1467,7 @@ class Holiday extends CommonObject
     function updateEventCP($rowid, $name, $value) {
 
         $sql = "UPDATE ".MAIN_DB_PREFIX."holiday_events SET";
-        $sql.= " name = '".addslashes($name)."', value = '".$value."'";
+        $sql.= " name = '".$this->db->escape($name)."', value = '".$value."'";
         $sql.= " WHERE rowid = '".$rowid."'";
 
         $result = $this->db->query($sql);
@@ -1548,7 +1590,7 @@ class Holiday extends CommonObject
      */
     function addLogCP($fk_user_action,$fk_user_update,$type,$new_solde) {
 
-        global $conf, $langs, $db;
+        global $conf, $langs;
 
         $error=0;
 
@@ -1570,7 +1612,7 @@ class Holiday extends CommonObject
         $sql.= " NOW(), ";
         $sql.= " '".$fk_user_action."',";
         $sql.= " '".$fk_user_update."',";
-        $sql.= " '".addslashes($type)."',";
+        $sql.= " '".$this->db->escape($type)."',";
         $sql.= " '".$prev_solde."',";
         $sql.= " '".$new_solde."'";
         $sql.= ")";
@@ -1628,7 +1670,7 @@ class Holiday extends CommonObject
         $sql.= " cpl.new_solde";
 
         $sql.= " FROM ".MAIN_DB_PREFIX."holiday_logs as cpl";
-        $sql.= " WHERE cpl.rowid > '0'"; // Hack pour la recherche sur le tableau
+        $sql.= " WHERE cpl.rowid > 0"; // To avoid error with other search and criteria
 
         // Filtrage de séléction
         if(!empty($filter)) {
@@ -1644,43 +1686,43 @@ class Holiday extends CommonObject
         $resql=$this->db->query($sql);
 
         // Si pas d'erreur SQL
-      		if ($resql) {
+  		if ($resql) {
 
-      		    $i = 0;
-      		    $tab_result = $this->logs;
-      		    $num = $this->db->num_rows($resql);
+  		    $i = 0;
+  		    $tab_result = $this->logs;
+  		    $num = $this->db->num_rows($resql);
 
-      		    // Si pas d'enregistrement
-      		    if(!$num) {
+  		    // Si pas d'enregistrement
+  		    if(!$num) {
                 return 2;
-      		    }
+  		    }
 
-      		    // On liste les résultats et on les ajoutent dans le tableau
-      		    while($i < $num) {
+  		    // On liste les résultats et on les ajoutent dans le tableau
+  		    while($i < $num) {
 
-      		        $obj = $this->db->fetch_object($resql);
+  		        $obj = $this->db->fetch_object($resql);
 
-      		        $tab_result[$i]['rowid'] = $obj->rowid;
-      		        $tab_result[$i]['date_action'] = $obj->date_action;
-      		        $tab_result[$i]['fk_user_action'] = $obj->fk_user_action;
-      		        $tab_result[$i]['fk_user_update'] = $obj->fk_user_update;
-      		        $tab_result[$i]['type_action'] = $obj->type_action;
-      		        $tab_result[$i]['prev_solde'] = $obj->prev_solde;
-      		        $tab_result[$i]['new_solde'] = $obj->new_solde;
+  		        $tab_result[$i]['rowid'] = $obj->rowid;
+  		        $tab_result[$i]['date_action'] = $obj->date_action;
+  		        $tab_result[$i]['fk_user_action'] = $obj->fk_user_action;
+  		        $tab_result[$i]['fk_user_update'] = $obj->fk_user_update;
+  		        $tab_result[$i]['type_action'] = $obj->type_action;
+  		        $tab_result[$i]['prev_solde'] = $obj->prev_solde;
+  		        $tab_result[$i]['new_solde'] = $obj->new_solde;
 
-      		        $i++;
-      		    }
-      		    // Retourne 1 et ajoute le tableau à la variable
-      		    $this->logs = $tab_result;
-      		    return 1;
-      		}
-      		else
-      		{
-      		    // Erreur SQL
-      		    $this->error="Error ".$this->db->lasterror();
-      		    dol_syslog(get_class($this)."::fetchLog ".$this->error, LOG_ERR);
-      		    return -1;
-      		}
+  		        $i++;
+  		    }
+  		    // Retourne 1 et ajoute le tableau à la variable
+  		    $this->logs = $tab_result;
+  		    return 1;
+  		}
+  		else
+  		{
+  		    // Erreur SQL
+  		    $this->error="Error ".$this->db->lasterror();
+  		    dol_syslog(get_class($this)."::fetchLog ".$this->error, LOG_ERR);
+  		    return -1;
+  		}
     }
 
     /**
