@@ -1,8 +1,8 @@
 <?php
-/* Copyright (C) 2011	Dimitri Mouillard	<dmouillard@teclib.com>
- * Copyright (C) 2012	Laurent Destailleur	<eldy@users.sourceforge.net>
- * Copyright (C) 2012	Regis Houssin		<regis.houssin@capnetworks.com>
- * Copyright (C) 2013	Juanjo Menent		<jmenent@2byte.es>
+/* Copyright (C) 2011	   Dimitri Mouillard	<dmouillard@teclib.com>
+ * Copyright (C) 2012-2013 Laurent Destailleur	<eldy@users.sourceforge.net>
+ * Copyright (C) 2012	   Regis Houssin		<regis.houssin@capnetworks.com>
+ * Copyright (C) 2013	   Juanjo Menent		<jmenent@2byte.es>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,9 +46,9 @@ $user_id = $user->id;
 $now=dol_now();
 
 
-/*******************************************************************
+/*
  * Actions
-********************************************************************/
+ */
 
 // Si création de la demande
 if ($action == 'create')
@@ -232,25 +232,41 @@ if ($action == 'update')
 }
 
 // Si suppression de la demande
-if ($action == 'confirm_delete'  && $_GET['confirm'] == 'yes')
+if ($action == 'confirm_delete' && GETPOST('confirm') == 'yes')
 {
     if($user->rights->holiday->delete)
     {
+    	$error=0;
+    	
+    	$db->begin();
+    	
         $cp = new Holiday($db);
-        $cp->fetch($_GET['id']);
+        $cp->fetch($id);
 
         // Si c'est bien un brouillon
-        if($cp->statut == 1) {
+        if ($cp->statut == 1 || $cp->statut == 3)
+        {
             // Si l'utilisateur à le droit de lire cette demande, il peut la supprimer
-            if($user->id == $cp->fk_user || $user->rights->holiday->lire_tous) {
-                $cp->delete($_GET['id']);
-                header('Location: index.php');
-                exit;
+            if ($user->id == $cp->fk_user || $user->rights->holiday->lire_tous)
+            {
+                $result=$cp->delete($id);
             }
-            else {
+            else 
+            {
                 $error = $langs->trans('ErrorCantDeleteCP');
             }
         }
+        
+       	if (! $error)
+        {
+          	$db->commit();
+           	header('Location: index.php');
+           	exit;
+        }
+        else
+        {
+        	$db->rollback();	
+        }
     }
 }
 
@@ -258,12 +274,10 @@ if ($action == 'confirm_delete'  && $_GET['confirm'] == 'yes')
 if ($action == 'confirm_send')
 {
     $cp = new Holiday($db);
-    $cp->fetch($_GET['id']);
-
-    $userID = $user->id;
+    $cp->fetch($id);
 
     // Si brouillon et créateur
-    if($cp->statut == 1 && $userID == $cp->fk_user)
+    if($cp->statut == 1 && $user->id == $cp->fk_user)
     {
         $cp->statut = 2;
 
@@ -295,7 +309,7 @@ if ($action == 'confirm_send')
             $subject = $societeName." - ".$langs->transnoentitiesnoconv("HolidaysToValidate");
 
             // Content
-            $message = $langs->transnoentitiesnoconv("Hello")." ".$destinataire->prenom.",\n";
+            $message = $langs->transnoentitiesnoconv("Hello")." ".$destinataire->firstname.",\n";
             $message.= "\n";
             $message.= $langs->transnoentities("HolidaysToValidateBody")."\n";
 
@@ -315,9 +329,9 @@ if ($action == 'confirm_send')
             }
 
             // Si l'option pour avertir le valideur en cas de solde inférieur à la demande
-            if($cp->getConfCP('AlertValidatorSolde'))
+            if ($cp->getConfCP('AlertValidatorSolde'))
             {
-            	$nbopenedday=num_open_day($cp->date_debut,$cp->date_fin,0,1);
+            	$nbopenedday=num_open_day($cp->date_debut,$cp->date_fin,0,1,$cp->halfday);
                 if ($nbopenedday > $cp->getCPforUser($cp->fk_user))
                 {
                     $message.= "\n";
@@ -326,7 +340,7 @@ if ($action == 'confirm_send')
             }
 
             $message.= "\n";
-            $message.= "- ".$langs->transnoentitiesnoconv("Name")." : ".$expediteur->prenom." ".$expediteur->nom."\n";
+            $message.= "- ".$langs->transnoentitiesnoconv("Name")." : ".dolGetFirstLastname($expediteur->firstname, $expediteur->lastname)."\n";
             $message.= "- ".$langs->transnoentitiesnoconv("Period")." : ".dol_print_date($cp->date_debut,'day')." ".$langs->transnoentitiesnoconv("To")." ".dol_print_date($cp->date_fin,'day')."\n";
             $message.= "- ".$langs->transnoentitiesnoconv("Link")." : ".$dolibarr_main_url_root."/holiday/fiche.php?id=".$cp->rowid."\n\n";
             $message.= "\n";
@@ -358,14 +372,11 @@ if ($action == 'confirm_send')
 if($action == 'confirm_valid')
 {
     $cp = new Holiday($db);
-    $cp->fetch($_GET['id']);
-
-    $userID = $user->id;
+    $cp->fetch($id);
 
     // Si statut en attente de validation et valideur = utilisateur
-    if($cp->statut == 2 && $userID == $cp->fk_validator)
+    if ($cp->statut == 2 && $user->id == $cp->fk_validator)
     {
-
         $cp->date_valid = dol_now();
         $cp->fk_user_valid = $user->id;
         $cp->statut = 3;
@@ -373,19 +384,19 @@ if($action == 'confirm_valid')
         $verif = $cp->update($user->id);
 
         // Si pas d'erreur SQL on redirige vers la fiche de la demande
-        if($verif > 0) {
-
-            // Retrait du nombre de jours prit
-            $nbJour = $nbopenedday=num_open_day($cp->date_debut,$cp->date_fin,0,1);
+        if ($verif > 0) 
+        {
+            // Calculcate number of days consummed
+            $nbopenedday=num_open_day($cp->date_debut,$cp->date_fin,0,1);
 
             $soldeActuel = $cp->getCpforUser($cp->fk_user);
-            $newSolde = $soldeActuel - ($nbJour*$cp->getConfCP('nbHolidayDeducted'));
+            $newSolde = $soldeActuel - ($nbopenedday * $cp->getConfCP('nbHolidayDeducted'));
 
             // On ajoute la modification dans le LOG
-            $cp->addLogCP($userID,$cp->fk_user, $langs->trans('Event').': '.$langs->transnoentitiesnoconv("Holidays"),$newSolde);
+            $cp->addLogCP($user->id, $cp->fk_user, $langs->transnoentitiesnoconv("Holidays"), $newSolde);
 
             // Mise à jour du solde
-            $cp->updateSoldeCP($cp->fk_user,$newSolde);
+            $cp->updateSoldeCP($cp->fk_user, $newSolde);
 
             // To
             $destinataire = new User($db);
@@ -410,10 +421,12 @@ if($action == 'confirm_valid')
             $subject = $societeName." - ".$langs->transnoentitiesnoconv("HolidaysValidated");
 
             // Content
-            $message = $langs->transnoentitiesnoconv("Hello")." ".$destinataire->prenom.",\n";
+            $message = $langs->transnoentitiesnoconv("Hello")." ".$destinataire->firstname.",\n";
             $message.= "\n";
             $message.=  $langs->transnoentities("HolidaysValidatedBody", dol_print_date($cp->date_debut,'day'),dol_print_date($cp->date_fin,'day'))."\n";
-            $message.= "- ".$langs->transnoentitiesnoconv("ValidatedBy")." : ".$expediteur->prenom." ".$expediteur->nom."\n";
+
+            $message.= "- ".$langs->transnoentitiesnoconv("ValidatedBy")." : ".dolGetFirstLastname($expediteur->firstname, $expediteur->lastname)."\n";
+
             $message.= "- ".$langs->transnoentitiesnoconv("Link")." : ".$dolibarr_main_url_root."/holiday/fiche.php?id=".$cp->rowid."\n\n";
             $message.= "\n";
 
@@ -441,16 +454,14 @@ if($action == 'confirm_valid')
 
 if ($action == 'confirm_refuse')
 {
-    if(!empty($_POST['detail_refuse']))
+    if (!empty($_POST['detail_refuse']))
     {
         $cp = new Holiday($db);
         $cp->fetch($_GET['id']);
 
-        $userID = $user->id;
-
         // Si statut en attente de validation et valideur = utilisateur
-        if($cp->statut == 2 && $userID == $cp->fk_validator) {
-
+        if ($cp->statut == 2 && $user->id == $cp->fk_validator)
+        {
             $cp->date_refuse = date('Y-m-d H:i:s', time());
             $cp->fk_user_refuse = $user->id;
             $cp->statut = 5;
@@ -459,8 +470,8 @@ if ($action == 'confirm_refuse')
             $verif = $cp->update($user->id);
 
             // Si pas d'erreur SQL on redirige vers la fiche de la demande
-            if($verif > 0) {
-
+            if ($verif > 0) 
+            {
                 // To
                 $destinataire = new User($db);
                 $destinataire->fetch($cp->fk_user);
@@ -484,12 +495,14 @@ if ($action == 'confirm_refuse')
 	            $subject = $societeName." - ".$langs->transnoentitiesnoconv("HolidaysRefused");
 
                 // Content
-            	$message = $langs->transnoentitiesnoconv("Hello")." ".$destinataire->prenom.",\n";
+            	$message = $langs->transnoentitiesnoconv("Hello")." ".$destinataire->firstname.",\n";
 	            $message.= "\n";
                 $message.= $langs->transnoentities("HolidaysRefusedBody", dol_print_date($cp->date_debut,'day'), dol_print_date($cp->date_fin,'day'))."\n";
                 $message.= GETPOST('detail_refuse','alpha')."\n\n";
-	            $message.= "- ".$langs->transnoentitiesnoconv("ModifiedBy")." : ".$expediteur->prenom." ".$expediteur->nom."\n"; 
-    	        $message.= "- ".$langs->transnoentitiesnoconv("Link")." : ".$dolibarr_main_url_root."/holiday/fiche.php?id=".$cp->rowid."\n\n";
+
+	            $message.= "- ".$langs->transnoentitiesnoconv("ModifiedBy")." : ".dolGetFirstLastname($expediteur->firstname, $expediteur->lastname)."\n";
+
+	            $message.= "- ".$langs->transnoentitiesnoconv("Link")." : ".$dolibarr_main_url_root."/holiday/fiche.php?id=".$cp->rowid."\n\n";
                 $message.= "\n";
 
                 $mail = new CMailFile($subject,$emailTo,$emailFrom,$message);
@@ -519,24 +532,54 @@ if ($action == 'confirm_refuse')
 }
 
 // Si Validation de la demande
-if ($action == 'confirm_cancel' && $_GET['confirm'] == 'yes')
+if ($action == 'confirm_cancel' && GETPOST('confirm') == 'yes')
 {
     $cp = new Holiday($db);
     $cp->fetch($_GET['id']);
 
-    $userID = $user->id;
-
     // Si statut en attente de validation et valideur = utilisateur
-    if ($cp->statut == 2 && $userID == $cp->fk_validator)
+    if (($cp->statut == 2 || $cp->statut == 3) && ($user->id == $cp->fk_validator || $user->id == $cp->fk_user))
     {
+    	$db->begin();
+    	
+    	$oldstatus = $cp->statut;
         $cp->date_cancel = dol_now();
         $cp->fk_user_cancel = $user->id;
         $cp->statut = 4;
 
-        $verif = $cp->update($user->id);
+        $result = $cp->update($user->id);
 
+        if ($result >= 0 && $oldstatus == 3)	// holiday was already validated, status 3, so we must increase back sold
+        {
+        	// Calculcate number of days consummed
+        	$nbopenedday=num_open_day($cp->date_debut,$cp->date_fin,0,1,$cp->halfday);
+
+        	$soldeActuel = $cp->getCpforUser($cp->fk_user);
+        	$newSolde = $soldeActuel + ($nbopenedday * $cp->getConfCP('nbHolidayDeducted'));
+
+        	// On ajoute la modification dans le LOG
+        	$result1=$cp->addLogCP($user->id, $cp->fk_user, $langs->transnoentitiesnoconv("HolidaysCancelation"), $newSolde);
+
+        	// Mise à jour du solde
+        	$result2=$cp->updateSoldeCP($cp->fk_user, $newSolde);
+
+        	if ($result1 < 0 || $result2 < 0)
+        	{
+        		$error = $langs->trans('ErrorCantDeleteCP');
+        	}
+        }
+        
+        if (! $error)
+        {
+        	$db->commit();	
+        }
+        else
+        {
+        	$db->rollback();
+        }
+        
         // Si pas d'erreur SQL on redirige vers la fiche de la demande
-        if($verif > 0)
+        if (! $error && $result > 0)
         {
             // To
             $destinataire = new User($db);
@@ -561,11 +604,13 @@ if ($action == 'confirm_cancel' && $_GET['confirm'] == 'yes')
             $subject = $societeName." - ".$langs->transnoentitiesnoconv("HolidaysCanceled");
 
             // Content
-           	$message = $langs->transnoentitiesnoconv("Hello")." ".$destinataire->prenom.",\n";
+           	$message = $langs->transnoentitiesnoconv("Hello")." ".$destinataire->firstname.",\n";
             $message.= "\n";
-         	$message.= $langs->transnoentities("HolidaysCanceledBody", dol_print_date($cp->date_debut,'day'), dol_print_date($cp->date_fin,'day'))."\n";
-            $message.= "- ".$langs->transnoentitiesnoconv("ModifiedBy")." : ".$expediteur->prenom." ".$expediteur->nom."\n";
-   	        $message.= "- ".$langs->transnoentitiesnoconv("Link")." : ".$dolibarr_main_url_root."/holiday/fiche.php?id=".$cp->rowid."\n\n";
+
+            $message.= $langs->transnoentities("HolidaysCanceledBody", dol_print_date($cp->date_debut,'day'), dol_print_date($cp->date_fin,'day'))."\n";
+            $message.= "- ".$langs->transnoentitiesnoconv("ModifiedBy")." : ".dolGetFirstLastname($expediteur->firstname, $expediteur->lastname)."\n";
+
+            $message.= "- ".$langs->transnoentitiesnoconv("Link")." : ".$dolibarr_main_url_root."/holiday/fiche.php?id=".$cp->rowid."\n\n";
             $message.= "\n";
 
             $mail = new CMailFile($subject,$emailTo,$emailFrom,$message);
@@ -638,7 +683,7 @@ if (empty($id) || $action == 'add' || $action == 'request')
                 case 'nodatedebut' :
                     $errors[] = $langs->trans('NoDateDebut');
                     break;
-                case 'nodatedebut' :
+                case 'nodatefin' :
                     $errors[] = $langs->trans('NoDateFin');
                     break;
                 case 'DureeHoliday' :
@@ -727,24 +772,29 @@ if (empty($id) || $action == 'add' || $action == 'request')
         print $form->selectarray('endhalfday', $listhalfday, (GETPOST('endhalfday')?GETPOST('endhalfday'):'afternoon'));
         print '</td>';
         print '</tr>';
+
+        // Approved by
         print '<tr>';
         print '<td class="fieldrequired">'.$langs->trans("ReviewedByCP").'</td>';
-        // Liste des utiliseurs du groupes choisi dans la config
-        $idGroupValid = $cp->getConfCP('userGroup');
-
-        $validator = new UserGroup($db, $idGroupValid);
-        $valideurarray = $validator->listUsersForGroup();
-
+        // Liste des utiliseurs du groupe choisi dans la config
+        $validator = new UserGroup($db);
+        $excludefilter=$user->admin?'':'u.rowid <> '.$user->id;
+        $valideurobjects = $validator->listUsersForGroup($excludefilter);
+        $valideurarray = array();
+        foreach($valideurobjects as $val) $valideurarray[$val->id]=$val->id;
         print '<td>';
-        print $form->select_dolusers($validator->id, "valideur", 1, "", 0, $valideurarray);
+        print $form->select_dolusers($user->fk_user, "valideur", 1, "", 0, $valideurarray);	// By default, hierarchical parent
         print '</td>';
         print '</tr>';
+
+        // Description
         print '<tr>';
         print '<td>'.$langs->trans("DescCP").'</td>';
         print '<td>';
         print '<textarea name="description" class="flat" rows="'.ROWS_3.'" cols="70"></textarea>';
         print '</td>';
         print '</tr>';
+
         print '</tbody>';
         print '</table>';
         print '<div style="clear: both;"></div>';
@@ -781,9 +831,6 @@ else
             $userRequest = new User($db);
             $userRequest->fetch($cp->fk_user);
 
-            // Utilisateur connecté
-            $userID = $user->id;
-
             //print_fiche_titre($langs->trans('TitreRequestCP'));
 
             // Si il y a une erreur
@@ -806,7 +853,7 @@ else
                     case 'nodatedebut' :
                         $errors[] = $langs->transnoentitiesnoconv('NoDateDebut');
                         break;
-                    case 'nodatedebut' :
+                    case 'nodatefin' :
                         $errors[] = $langs->transnoentitiesnoconv('NoDateFin');
                         break;
                     case 'DureeHoliday' :
@@ -827,41 +874,37 @@ else
             if($user->id == $cp->fk_user || $user->rights->holiday->lire_tous)
             {
 
-                if ($action == 'delete' && $cp->statut == 1) {
+                if ($action == 'delete') 
+                {
                     if($user->rights->holiday->delete)
                     {
-                        $ret=$form->form_confirm("fiche.php?id=".$id,$langs->trans("TitleDeleteCP"),$langs->trans("ConfirmDeleteCP"),"confirm_delete", '', 0, 1);
-                        if ($ret == 'html') print '<br />';
+                        print $form->formconfirm("fiche.php?id=".$id,$langs->trans("TitleDeleteCP"),$langs->trans("ConfirmDeleteCP"),"confirm_delete", '', 0, 1);
                     }
                 }
 
                 // Si envoi en validation
-                if ($action == 'sendToValidate' && $cp->statut == 1 && $userID == $cp->fk_user)
+                if ($action == 'sendToValidate' && $cp->statut == 1 && $user->id == $cp->fk_user)
                 {
-                    $ret=$form->form_confirm("fiche.php?id=".$id,$langs->trans("TitleToValidCP"),$langs->trans("ConfirmToValidCP"),"confirm_send", '', 1, 1);
-                    if ($ret == 'html') print '<br />';
+                    print $form->formconfirm("fiche.php?id=".$id,$langs->trans("TitleToValidCP"),$langs->trans("ConfirmToValidCP"),"confirm_send", '', 1, 1);
                 }
 
                 // Si validation de la demande
-                if ($action == 'valid' && $cp->statut == 2 && $userID == $cp->fk_validator)
+                if ($action == 'valid')
                 {
-                    $ret=$form->form_confirm("fiche.php?id=".$id,$langs->trans("TitleValidCP"),$langs->trans("ConfirmValidCP"),"confirm_valid", '', 1, 1);
-                    if ($ret == 'html') print '<br />';
+                    print $form->formconfirm("fiche.php?id=".$id,$langs->trans("TitleValidCP"),$langs->trans("ConfirmValidCP"),"confirm_valid", '', 1, 1);
                 }
 
                 // Si refus de la demande
-                if ($action == 'refuse' && $cp->statut == 2 && $userID == $cp->fk_validator)
+                if ($action == 'refuse')
                 {
-                    $array_input = array(array('type'=>"text",'label'=>"Entrez ci-dessous un motif de refus :",'name'=>"detail_refuse",'size'=>"50",'value'=>""));
-                    $ret=$form->form_confirm("fiche.php?id=".$id."&action=confirm_refuse", $langs->trans("TitleRefuseCP"), "", "confirm_refuse", $array_input, 1, 0);
-                    if ($ret == 'html') print '<br />';
+                    $array_input = array(array('type'=>"text",'label'=> $langs->trans('DetailRefusCP'),'name'=>"detail_refuse",'size'=>"50",'value'=>""));
+                    print $form->formconfirm("fiche.php?id=".$id."&action=confirm_refuse", $langs->trans("TitleRefuseCP"), $langs->trans('ConfirmRefuseCP'), "confirm_refuse", $array_input, 1, 0);
                 }
 
                 // Si annulation de la demande
-                if ($action == 'cancel' && $cp->statut == 2 && $userID == $cp->fk_validator)
+                if ($action == 'cancel')
                 {
-                    $ret=$form->form_confirm("fiche.php?id=".$id,$langs->trans("TitleCancelCP"),$langs->trans("ConfirmCancelCP"),"confirm_cancel", '', 1, 1);
-                    if ($ret == 'html') print '<br />';
+                    print $form->formconfirm("fiche.php?id=".$id,$langs->trans("TitleCancelCP"),$langs->trans("ConfirmCancelCP"),"confirm_cancel", '', 1, 1);
                 }
 
                 $head=holiday_prepare_head($cp);
@@ -891,7 +934,8 @@ else
 			    $starthalfday=($cp->halfday == -1 || $cp->halfday == 2)?'afternoon':'morning';
 			    $endhalfday=($cp->halfday == 1 || $cp->halfday == 2)?'morning':'afternoon';
 
-                if(!$edit) {
+                if(!$edit)
+                {
                     print '<tr>';
                     print '<td>'.$langs->trans('DateDebCP').' ('.$langs->trans("FirstDayOfHoliday").')</td>';
                     print '<td>'.dol_print_date($cp->date_debut,'day');
@@ -899,7 +943,9 @@ else
                     print $langs->trans($listhalfday[$starthalfday]);
                     print '</td>';
                     print '</tr>';
-                } else {
+                } 
+                else 
+                {
                     print '<tr>';
                     print '<td>'.$langs->trans('DateDebCP').' ('.$langs->trans("FirstDayOfHoliday").')</td>';
                     print '<td>';
@@ -919,7 +965,9 @@ else
                     print $langs->trans($listhalfday[$endhalfday]);
                     print '</td>';
                     print '</tr>';
-                } else {
+                } 
+                else 
+                {
                     print '<tr>';
                     print '<td>'.$langs->trans('DateFinCP').' ('.$langs->trans("LastDayOfHoliday").')</td>';
                     print '<td>';
@@ -991,7 +1039,7 @@ else
                     $idGroupValid = $cp->getConfCP('userGroup');
 
                     $validator = new UserGroup($db,$idGroupValid);
-                    $valideur = $validator->listUsersForGroup();
+                    $valideur = $validator->listUsersForGroup('',1);
 
                     print '<td>';
                     $form->select_users($cp->fk_validator,"valideur",1,"",0,$valideur,'');
@@ -1051,21 +1099,21 @@ else
                     {
                         print '<a href="fiche.php?id='.$_GET['id'].'&action=sendToValidate" class="butAction">'.$langs->trans("Validate").'</a>';
                     }
-                    if($user->rights->holiday->delete && $cp->statut == 1)
+                    if ($user->rights->holiday->delete && $cp->statut == 1)	// If draft
                     {
                     	print '<a href="fiche.php?id='.$_GET['id'].'&action=delete" class="butActionDelete">'.$langs->trans("DeleteCP").'</a>';
                     }
 
-                    // Si le statut est en attente de validation et que le valideur est connecté
-                    if ($userID == $cp->fk_validator && $cp->statut == 2)
+                    if ($user->id == $cp->fk_validator && $cp->statut == 2)
                     {
                         print '<a href="fiche.php?id='.$_GET['id'].'&action=valid" class="butAction">'.$langs->trans("Approve").'</a>';
                         print '<a href="fiche.php?id='.$_GET['id'].'&action=refuse" class="butAction">'.$langs->trans("ActionRefuseCP").'</a>';
                     }
 
-                    if (($userID == $cp->fk_validator && $cp->statut == 2) || ($cp->date_debut > dol_now()) || $user->admin)
+                    if (($user->id == $cp->fk_validator || $user->id == $cp->fk_user) && ($cp->statut == 2 || $cp->statut == 3))	// Status validated or approved
                     {
-	                    print '<a href="fiche.php?id='.$_GET['id'].'&action=cancel" class="butAction">'.$langs->trans("ActionCancelCP").'</a>';
+                    	if (($cp->date_debut > dol_now()) || $user->admin) print '<a href="fiche.php?id='.$_GET['id'].'&action=cancel" class="butAction">'.$langs->trans("ActionCancelCP").'</a>';
+                    	else print '<a href="#" class="butActionRefused" title="'.$langs->trans("HolidayStarted").'">'.$langs->trans("ActionCancelCP").'</a>';
                     }
 
                     print '</div>';
