@@ -64,9 +64,6 @@ if ($action == 'add' && $user->rights->stock->creer)
 	$object->statut      = $_POST["statut"];
 	$object->lieu        = $_POST["lieu"];
 	$object->address     = $_POST["address"];
-	$object->cp          = $_POST["zipcode"];
-	$object->ville       = $_POST["town"];
-	$object->pays_id     = $_POST["country_id"];
 	$object->zip         = $_POST["zipcode"];
 	$object->town        = $_POST["town"];
 	$object->country_id  = $_POST["country_id"];
@@ -117,9 +114,6 @@ if ($action == 'update' && $_POST["cancel"] <> $langs->trans("Cancel"))
 		$object->statut      = $_POST["statut"];
 		$object->lieu        = $_POST["lieu"];
 		$object->address     = $_POST["address"];
-		$object->cp          = $_POST["zipcode"];
-		$object->ville       = $_POST["town"];
-		$object->pays_id     = $_POST["country_id"];
 		$object->zip         = $_POST["zipcode"];
 		$object->town        = $_POST["town"];
 		$object->country_id  = $_POST["country_id"];
@@ -197,9 +191,9 @@ if ($action == 'create')
 
 	// Zip / Town
 	print '<tr><td>'.$langs->trans('Zip').'</td><td>';
-	print $formcompany->select_ziptown($object->zip,'zipcode',array('town','selectcountry_id','departement_id'),6);
+	print $formcompany->select_ziptown($object->zip,'zipcode',array('town','selectcountry_id','state_id'),6);
 	print '</td><td>'.$langs->trans('Town').'</td><td>';
-	print $formcompany->select_ziptown($object->town,'town',array('zipcode','selectcountry_id','departement_id'));
+	print $formcompany->select_ziptown($object->town,'town',array('zipcode','selectcountry_id','state_id'));
 	print '</td></tr>';
 
 	// Country
@@ -246,9 +240,7 @@ else
 			// Confirm delete third party
 			if ($action == 'delete')
 			{
-				$form = new Form($db);
-				$ret=$form->form_confirm($_SERVER["PHP_SELF"]."?id=".$object->id,$langs->trans("DeleteAWarehouse"),$langs->trans("ConfirmDeleteWarehouse",$object->libelle),"confirm_delete",'',0,2);
-				if ($ret == 'html') print '<br>';
+				print $form->formconfirm($_SERVER["PHP_SELF"]."?id=".$object->id,$langs->trans("DeleteAWarehouse"),$langs->trans("ConfirmDeleteWarehouse",$object->libelle),"confirm_delete",'',0,2);
 			}
 
 			print '<table class="border" width="100%">';
@@ -270,22 +262,31 @@ else
 			print $object->address;
 			print '</td></tr>';
 
-			// Ville
+			// Town
 			print '<tr><td width="25%">'.$langs->trans('Zip').'</td><td width="25%">'.$object->zip.'</td>';
 			print '<td width="25%">'.$langs->trans('Town').'</td><td width="25%">'.$object->town.'</td></tr>';
 
 			// Country
 			print '<tr><td>'.$langs->trans('Country').'</td><td colspan="3">';
-			$img=picto_from_langcode($object->country_code);
-			print ($img?$img.' ':'');
+			if (! empty($object->country_code)) 
+			{
+				$img=picto_from_langcode($object->country_code);
+				print ($img?$img.' ':'');
+			}
 			print $object->country;
 			print '</td></tr>';
 
-			// Statut
+			// Status
 			print '<tr><td>'.$langs->trans("Status").'</td><td colspan="3">'.$object->getLibStatut(4).'</td></tr>';
 
+        	$calcproductsunique=$object->nb_different_products();
 			$calcproducts=$object->nb_products();
 
+	        // Total nb of different products
+	        print '<tr><td valign="top">'.$langs->trans("NumberOfDifferentProducts").'</td><td colspan="3">';
+	        print empty($calcproductsunique['nb'])?'0':$calcproductsunique['nb'];
+	        print "</td></tr>";
+
 			// Nb of products
 			print '<tr><td valign="top">'.$langs->trans("NumberOfProducts").'</td><td colspan="3">';
 			print empty($calcproducts['nb'])?'0':$calcproducts['nb'];
@@ -518,9 +519,9 @@ else
 
 			// Zip / Town
 			print '<tr><td>'.$langs->trans('Zip').'</td><td>';
-			print $formcompany->select_ziptown($object->zip,'zipcode',array('town','selectcountry_id','departement_id'),6);
+			print $formcompany->select_ziptown($object->zip,'zipcode',array('town','selectcountry_id','state_id'),6);
 			print '</td><td>'.$langs->trans('Town').'</td><td>';
-			print $formcompany->select_ziptown($object->town,'town',array('zipcode','selectcountry_id','departement_id'));
+			print $formcompany->select_ziptown($object->town,'town',array('zipcode','selectcountry_id','state_id'));
 			print '</td></tr>';
 
 			// Country
